from django.contrib.auth.models import User
from rest_framework import viewsets

from .models import (
    InstitutionInstitutionRelation, TextType, CollectionType, VocabsBaseClass,
    InstitutionType, ProfessionType, PlaceType, PersonInstitutionRelation, PersonPlaceRelation, PersonPersonRelation,
    VocabNames, InstitutionPlaceRelation, PersonEventRelation,
    PersonWorkRelation, InstitutionEventRelation, InstitutionWorkRelation, PlaceWorkRelation, PlaceEventRelation,
    PlacePlaceRelation, EventWorkRelation, EventEventRelation, WorkWorkRelation, EventType, WorkType)
from .serializers import (
    InstitutionInstitutionRelationSerializer, TextTypeSerializer,
    CollectionTypeSerializer, VocabsBaseClassSerializer,
    InstitutionTypeSerializer, ProfessionTypeSerializer, InstitutionPlaceRelationSerializer,
    PlaceTypeSerializer, PersonInstitutionRelationSerializer,
    PersonPlaceRelationSerializer, UserAccSerializer, VocabNamesSerializer,
    PersonPersonRelationSerializer, PersonEventRelationSerializer, PersonWorkRelationSerializer,
    InstitutionEventRelationSerializer, InstitutionWorkRelationSerializer, PlaceEventRelationSerializer,
    PlaceWorkRelationSerializer, PlacePlaceRelationSerializer, EventWorkRelationSerializer,
    EventEventRelationSerializer, WorkWorkRelationSerializer, EventTypeSerializer, WorkTypeSerializer)


###########################################################
#
# Meta - ViewSets
#
##########################################################


class UserViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = User.objects.all()
    serializer_class = UserAccSerializer


class VocabNamesViewSet(viewsets.ModelViewSet):
    queryset = VocabNames.objects.all()
    serializer_class = VocabNamesSerializer


class CollectionTypeViewSet(viewsets.ModelViewSet):
    queryset = CollectionType.objects.all()
    serializer_class = CollectionTypeSerializer


class VocabsBaseClassViewSet(viewsets.ModelViewSet):
    queryset = VocabsBaseClass.objects.all()
    serializer_class = VocabsBaseClassSerializer


########################################################
#
# Entity Types
#
#######################################################


class TextTypeViewSet(viewsets.ModelViewSet):
    queryset = TextType.objects.all()
    serializer_class = TextTypeSerializer


class InstitutionTypeViewSet(viewsets.ModelViewSet):
    queryset = InstitutionType.objects.all()
    serializer_class = InstitutionTypeSerializer


class ProfessionTypeViewSet(viewsets.ModelViewSet):
    queryset = ProfessionType.objects.all()
    serializer_class = ProfessionTypeSerializer


class PlaceTypeViewSet(viewsets.ModelViewSet):
    queryset = PlaceType.objects.all()
    serializer_class = PlaceTypeSerializer


class EventTypeViewSet(viewsets.ModelViewSet):
    queryset = EventType.objects.all()
    serializer_class = EventTypeSerializer


class WorkTypeViewSet(viewsets.ModelViewSet):
    queryset = WorkType.objects.all()
    serializer_class = WorkTypeSerializer


######################################################
#
# Relation Types
#
#####################################################


class PersonInstitutionRelationViewSet(viewsets.ModelViewSet):
    queryset = PersonInstitutionRelation.objects.all()
    serializer_class = PersonInstitutionRelationSerializer


class PersonPlaceRelationViewSet(viewsets.ModelViewSet):
    queryset = PersonPlaceRelation.objects.all()
    serializer_class = PersonPlaceRelationSerializer


class PersonEventRelationViewSet(viewsets.ModelViewSet):
    queryset = PersonEventRelation.objects.all()
    serializer_class = PersonEventRelationSerializer


class PersonWorkRelationViewSet(viewsets.ModelViewSet):
    queryset = PersonWorkRelation.objects.all()
    serializer_class = PersonWorkRelationSerializer


class PersonPersonRelationViewSet(viewsets.ModelViewSet):
    queryset = PersonPersonRelation.objects.all()
    serializer_class = PersonPersonRelationSerializer


class InstitutionInstitutionRelationViewSet(viewsets.ModelViewSet):
    queryset = InstitutionInstitutionRelation.objects.all()
    serializer_class = InstitutionInstitutionRelationSerializer


class InstitutionPlaceRelationViewSet(viewsets.ModelViewSet):
    queryset = InstitutionPlaceRelation.objects.all()
    serializer_class = InstitutionPlaceRelationSerializer


class InstitutionEventRelationViewSet(viewsets.ModelViewSet):
    queryset = InstitutionEventRelation.objects.all()
    serializer_class = InstitutionEventRelationSerializer


class InstitutionWorkRelationViewSet(viewsets.ModelViewSet):
    queryset = InstitutionWorkRelation.objects.all()
    serializer_class = InstitutionWorkRelationSerializer


class PlaceEventRelationViewSet(viewsets.ModelViewSet):
    queryset = PlaceEventRelation.objects.all()
    serializer_class = PlaceEventRelationSerializer


class PlaceWorkRelationViewSet(viewsets.ModelViewSet):
    queryset = PlaceWorkRelation.objects.all()
    serializer_class = PlaceWorkRelationSerializer


class PlacePlaceRelationViewSet(viewsets.ModelViewSet):
    queryset = PlacePlaceRelation.objects.all()
    serializer_class = PlacePlaceRelationSerializer


class EventWorkRelationViewSet(viewsets.ModelViewSet):
    queryset = EventWorkRelation.objects.all()
    serializer_class = EventWorkRelationSerializer


class EventEventRelationViewSet(viewsets.ModelViewSet):
    queryset = EventEventRelation.objects.all()
    serializer_class = EventEventRelationSerializer


class WorkWorkRelationViewSet(viewsets.ModelViewSet):
    queryset = WorkWorkRelation.objects.all()
    serializer_class = WorkWorkRelationSerializer






import unicodedata

# from metainfo.models import TempEntityClass
# from reversion import revisions as reversion
import reversion
from django.db import models

from apis_core.apis_vocabularies.models import LabelType
from apis_core.helper_functions import DateParser


@reversion.register()
class Label(models.Model):
    label = models.CharField(
        max_length=255, blank=True, help_text="The entities label or name.")
    isoCode_639_3 = models.CharField(
        max_length=3, blank=True, null=True,
        help_text="The ISO 639-3 (or 2) code for the label's language.",
        verbose_name='ISO Code', default='deu')
    label_type = models.ForeignKey(LabelType, blank=True, null=True,
                                   on_delete=models.SET_NULL)

    start_date = models.DateField(blank=True, null=True)
    start_start_date = models.DateField(blank=True, null=True)
    start_end_date = models.DateField(blank=True, null=True)
    end_date = models.DateField(blank=True, null=True)
    end_start_date = models.DateField(blank=True, null=True)
    end_end_date = models.DateField(blank=True, null=True)
    start_date_written = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name="Start",
    )
    end_date_written = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name="End",
    )

    # TODO __sresch__ add related_name="label_set" here to be consistent with other usages throughout django
    temp_entity = models.ForeignKey("apis_metainfo.TempEntityClass", on_delete=models.CASCADE)

    def get_web_object(self):
        result = {
            'relation_pk': self.pk,
            'label': self.label,
            'isoCode_639_3': self.isoCode_639_3,
            'label_type': self.label_type.name}
        return result

    def save(self, *args, **kwargs):
        if self.label != unicodedata.normalize('NFC', self.label):    #secure correct unicode encoding
            self.label = unicodedata.normalize('NFC', self.label)

        def parse_dates():

            # overwrite every field with None as default
            start_date = None
            start_start_date = None
            start_end_date = None
            end_date = None
            end_start_date = None
            end_end_date = None

            if self.start_date_written:
                # If some textual user input of a start date is there, then parse it

                start_date, start_start_date, start_end_date = \
                    DateParser.parse_date(self.start_date_written)

            if self.end_date_written:
                # If some textual user input of an end date is there, then parse it

                end_date, end_start_date, end_end_date = \
                    DateParser.parse_date(self.end_date_written)

            self.start_date = start_date
            self.start_start_date = start_start_date
            self.start_end_date = start_end_date
            self.end_date = end_date
            self.end_start_date = end_start_date
            self.end_end_date = end_end_date

        parse_dates()

        super(Label, self).save(*args, **kwargs)
        return self

    def __str__(self):
        return self.label



import time

from django.test import TestCase

from apis_core.apis_relations.models import PersonPlace, InstitutionPlace
from apis_core.helper_functions.RDFParser import RDFParser


class RDFPlaceParserNewTestCase(TestCase):

    def test_parse_place(self):
        plc = [("http://www.geonames.org/6951114/bad-zell.html", "Bad Zell", 48.34906, "http://sws.geonames.org/2782113/", "https://www.geonames.org/ontology#P.PPL"),
               ("http://sws.geonames.org/2779275/", "Freistadt", 48.51103, "http://sws.geonames.org/2782113/", "https://www.geonames.org/ontology#P.PPL"),
               ("http://sws.geonames.org/2761369", "Vienna", 48.20849, "http://sws.geonames.org/2782113/", "https://www.geonames.org/ontology#P.PPLC")
               ]
        for p in plc:
            print(p)
            o = RDFParser(p[0], 'Place')
            o.create_objct()
            o2 = o.save()
            print(type(o2.lat))
            self.assertEqual(o2.lat, float(p[2]))
            self.assertEqual(o2.name, p[1])
            self.assertEqual(o2.kind.name, p[4])



    #def test_parse_person(self):
     #   o = RDFParser('http://d-nb.info/gnd/118650130', 'Person', uri_check=False)
      #  o.create_objct()
       # print(o._foreign_keys)
        #print(o.objct)
       # print(o._attributes)
       # print(o.objct.name)
       # print(o._foreign_keys)
       # self.assertEqual(o.objct.name, 'Aristoteles')
       # o2 = o.save()
       # print(o2)
       # o3 = RDFParser('http://d-nb.info/gnd/118566512', 'Person', uri_check=False)
       # o3.create_objct()
       # print(o3._foreign_keys)
       # print(o3.objct)
       # print(o3._attributes)
       # print(f"name: {o3.objct.name}, {o3.objct.first_name}")
       # print(o3._foreign_keys)
       # self.assertEqual(o3.objct.name, 'Kreisky')
       # self.assertEqual(o3.objct.start_date_written, '1911-01-22')
       # o4 = o3.save()
       # print(o4)
       # print(o4.profession.all())
       # print(o4.start_date)

    def test_merge(self):
        o = RDFParser('http://d-nb.info/gnd/118566512', 'Person', uri_check=False)
        o.create_objct()
        o.save()
        o2 = RDFParser('http://d-nb.info/gnd/170686299', 'Person', uri_check=False).get_or_create()
        o.merge(o2)
        print(o.objct.uri_set.all())

    def test_place_of_birth(self):
        o = RDFParser('http://d-nb.info/gnd/118566512', 'Person', uri_check=False)
        o.create_objct()
        o.save()
        for p in PersonPlace.objects.all():
            print(p.related_place.name, p.related_place.uri_set.all(), p.related_place.lat)

    def test_institution(self):
        o = RDFParser('http://d-nb.info/gnd/1001454-8', 'Institution', uri_check=False)
        o.create_objct()
        o2 = o.save()
        print(o2.pk, o2.start_date, o2.start_date_written)
        print(InstitutionPlace.objects.filter(related_institution_id=o2.pk))
        print(o2.label_set.all().values_list('label'))
        
    def test_use_uri_twice(self):
        self.assertRaises(ValueError, RDFParser, "http://www.geonames.org/6951114/bad-zell.html", "Place")

    def test_uri_twice_later(self):
        o = RDFParser('https://sws.geonames.org/2778843', 'Place', preserve_uri_minutes=0.5)
        self.assertRaises(ValueError, RDFParser, 'https://sws.geonames.org/2778843', "Place", preserve_uri_minutes=0.5)
        time.sleep(35)
        o2 = RDFParser('https://sws.geonames.org/2778843', 'Place', preserve_uri_minutes=0.5).get_or_create()

    def test_institution_government(self):
        o = RDFParser('https://d-nb.info/gnd/2029534-0', 'Institution').get_or_create()
        self.assertEqual(o.name, 'Österreich. Bundesregierung')
        #self.assertEqual(o.start_date_written, '1945')

    def test_uni_koeln(self):
        o = RDFParser('https://d-nb.info/gnd/2024231-1', 'Institution')
        o.create_objct()
        print(o._foreign_keys)
        o2 = o.save()
        for p in InstitutionPlace.objects.filter(related_institution_id=o2.pk):
            print(p.related_place, p.related_place.kind)



from django.conf import settings
from rdflib import Graph, URIRef, Literal
from rdflib.namespace import SKOS, RDF, DC, RDFS
from rest_framework import renderers

try:
    from webpage.metadata import PROJECT_METADATA
except ImportError:
    from webpage.utils import PROJECT_METADATA


base_uri_web = getattr(settings, 'APIS_BASE_URI', 'http://apis.info')
if base_uri_web.endswith('/'):
    base_uri_web = base_uri_web[:-1]
lang = getattr(settings, 'LANGUAGE_CODE', 'de')


class VocabToSkos(renderers.BaseRenderer):

    media_type = "text/rdf"

    def render(self, data1, media_type=None, g=None, renderer_context=None, format1=None, binary=True, store=None):
        if g is None:
            g = Graph()
        set_skos = getattr(settings, "APIS_SKOSMOS")
        base_uri = set_skos.get('url')
        if base_uri.endswith('/'):
            base_uri = base_uri[:-1]
        title = PROJECT_METADATA.get('title', 'TITLE')
        v_title = set_skos.get("vocabs-name", False)
        if not v_title:
            v_title = f"{title.lower()}thesaurus/"
        uri_1 = f"{base_uri}/{v_title}"
        uri = URIRef(f"{uri_1}/{title.title()}Schema")
        g.add((uri, RDFS.label, Literal(f"{title}Schema", lang=lang)))
        g.add((RDF.Description, RDF.about, uri))
        g.add((uri, RDF.type, SKOS.ConceptScheme))
        g.add((uri, DC.description, Literal(f"{set_skos.get('description', 'DESCRIPTION')}", lang=lang)))
        g.add((uri, DC.license, URIRef(set_skos.get("license", "https://opensource.org/licenses/MIT"))))
        g.add((uri, DC.relation, URIRef(base_uri_web)))
        
        cols = {}
        for d in data1:
            if not cols.get(d["vocab_name"], False):
                vc = URIRef(f"{uri_1}/{d['vocab_name']}")
                g.add((vc, RDF.type, SKOS.Collection))
                g.add((vc, SKOS.prefLabel, Literal(d['vocab_name'], lang=lang)))
                cols[d["vocab_name"]] = vc
            else:
                vc = cols.get(d["vocab_name"])
            conc = URIRef(f"{uri_1}/{d['id']}")
            g.add((vc, SKOS.member, conc))
            g.add((conc, RDF.type, SKOS.Concept))
            g.add((conc, SKOS.prefLabel, Literal(d['name'], lang=lang)))
            if d['name'] != d['label']:
                g.add((conc, SKOS.altLabel, Literal(d['label'], lang=lang)))
            rev = d.get('name_reverse', False)
            if rev:
                g.add((conc, SKOS.altLabel, Literal(f"reverse name: {rev}", lang=lang)))
                if d['name_reverse'] != d['label_reverse']:
                    g.add((conc, SKOS.altLabel, Literal(f"reverse label: {d['label_reverse']}", lang=lang)))
            if d.get('userAdded', False):
                if (uri, DC.creator, Literal(d['userAdded'], lang=lang)) not in g:
                    g.add((uri, DC.creator, Literal(d['userAdded'], lang=lang)))
            broader = d.get('parent_class', False)
            if broader:
                g.add((conc, SKOS.broader, URIRef(f"{uri_1}/{broader}")))
            else:
                g.add((conc, SKOS.topConceptOf, uri))
                g.add((uri, SKOS.hasTopConcept, conc))
        if format1:
            return g.serialize(format1)
        elif binary:
            return g, store



import django_filters

from .models import *


class UriListFilter(django_filters.FilterSet):

    uri = django_filters.CharFilter(
        lookup_expr='icontains',
        help_text=Uri._meta.get_field('uri').help_text,
        label=Uri._meta.get_field('uri').verbose_name
        )
    domain = django_filters.CharFilter(
        lookup_expr='icontains',
        help_text=Uri._meta.get_field('domain').help_text,
        label=Uri._meta.get_field('domain').verbose_name
        )
    entity__name = django_filters.CharFilter(
        lookup_expr='icontains',
        help_text=Uri._meta.get_field('entity').help_text,
        label=Uri._meta.get_field('entity').verbose_name
        )

    class Meta:
        model = Uri
        fields = "__all__"



import django_tables2 as tables
from django.utils.safestring import mark_safe
from django_tables2.utils import A

from apis_core.apis_entities.models import AbstractEntity
from apis_core.apis_metainfo.tables import (
    generic_order_start_date_written,
    generic_order_end_date_written,
    generic_render_start_date_written,
    generic_render_end_date_written
)

input_form = """
  <input type="checkbox" name="keep" value="{}" title="keep this"/> |
  <input type="checkbox" name="remove" value="{}" title="remove this"/>
"""


class MergeColumn(tables.Column):
    """ renders a column with to checkbox - used to select objects for merging """

    def __init__(self, *args, **kwargs):
        super(MergeColumn, self).__init__(*args, **kwargs)

    def render(self, value):
        return mark_safe(
            input_form.format(value, value)
        )


def get_entities_table(entity, edit_v, default_cols):

    if default_cols is None:
        default_cols = ['name', ]

    class GenericEntitiesTable(tables.Table):

        def render_name(self, record, value):
            if value == "":
                return "(No name provided)"
            else:
                return value

        # reuse the logic for ordering and rendering *_date_written
        # Important: The names of these class variables must correspond to the column field name,
        # e.g. for start_date_written, the methods must be named order_start_date_written and render_start_date_written
        order_start_date_written = generic_order_start_date_written
        order_end_date_written = generic_order_end_date_written
        render_start_date_written = generic_render_start_date_written
        render_end_date_written = generic_render_end_date_written

        if edit_v:
            name = tables.LinkColumn(
                'apis:apis_entities:generic_entities_edit_view',
                args=[entity.lower(), A('pk')],
                empty_values=[]
            )
        else:
            name = tables.LinkColumn(
                'apis:apis_entities:generic_entities_detail_view',
                args=[entity.lower(), A('pk')],
                empty_values=[]
            )
        export_formats = [
            'csv',
            'json',
            'xls',
            'xlsx',
        ]
        if 'merge' in default_cols:
            merge = MergeColumn(verbose_name='keep | remove', accessor='pk')
        if 'id' in default_cols:
            id = tables.LinkColumn()

        class Meta:
            model = AbstractEntity.get_entity_class_of_name(entity_name=entity)

            fields = default_cols
            attrs = {"class": "table table-hover table-striped table-condensed"}

            # quick ensurance if column is indeed a field of this entity
            for col in default_cols:
                if not hasattr(model, col):
                    raise Exception(
                        f"Could not find field in entity: {entity}\n"
                        f"of column (probably defined in 'table_fields' settings): {col}\n"
                    )

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

    return GenericEntitiesTable





from django.conf import settings
from django.views.generic.detail import DetailView
from django_tables2 import RequestConfig

from apis_core.apis_entities.views import GenericListViewNew
from apis_core.apis_relations.models import AbstractRelation
from .rel_filters import get_generic_relation_filter
from .tables import get_generic_relation_listview_table


class GenericRelationView(GenericListViewNew):

    context_filter_name = 'filter'
    paginate_by = 25
    template_name = getattr(
        settings,
        'APIS_LIST_VIEW_TEMPLATE',
        'apis_entities/generic_list.html'
    )
    login_url = '/accounts/login/'

    def get_queryset(self, **kwargs):
        self.entity = self.kwargs.get('entity')
        qs = AbstractRelation.get_relation_class_of_name(self.entity).objects.all()
        self.filter = get_generic_relation_filter(
            self.entity.title())(self.request.GET, queryset=qs)
        self.filter.form.helper = self.formhelper_class()
        if callable(getattr(self.filter.qs, 'filter_for_user', None)):
            return self.filter.qs.filter_for_user().distinct()
        else:
            return self.filter.qs.distinct()


    def get_table(self, **kwargs):
        relation_name = self.kwargs['entity'].lower()
        self.table_class = get_generic_relation_listview_table(relation_name=relation_name)
        table = super(GenericListViewNew, self).get_table()
        RequestConfig(self.request, paginate={
            'page': 1, 'per_page': self.paginate_by}).configure(table)
        return table


class GenericRelationDetailView(DetailView):

    template_name = getattr(
        settings,
        'APIS_RELATIONS_DETAIL_VIEW_TEMPLATE',
        'apis_relations/relations_detail_generic.html'
    )

    def get_object(self):
        entity = self.kwargs['entity'].lower()
        instance = self.kwargs['pk'].lower()
        entity_model = AbstractRelation.get_relation_class_of_name(entity)
        instance = entity_model.objects.get(pk=instance)
        return instance

    def get_context_data(self, **kwargs):
        context = super(GenericRelationDetailView, self).get_context_data()
        context['entity'] = self.kwargs['entity'].lower()
        context['entity_type'] = context['entity']
        return context



# Generated by Django 2.1.12 on 2020-01-21 12:27

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_vocabularies', '0001_initial'),
        ('apis_relations', '0001_initial'),
        ('apis_entities', '0002_auto_20200121_1227'),
    ]

    operations = [
        migrations.AddField(
            model_name='workwork',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='workwork_set', to='apis_vocabularies.WorkWorkRelation'),
        ),
        migrations.AddField(
            model_name='placework',
            name='related_place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='placework_set', to='apis_entities.Place'),
        ),
        migrations.AddField(
            model_name='placework',
            name='related_work',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='placework_set', to='apis_entities.Work'),
        ),
        migrations.AddField(
            model_name='placework',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='placework_set', to='apis_vocabularies.PlaceWorkRelation'),
        ),
        migrations.AddField(
            model_name='placeplace',
            name='related_placeA',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_placeB', to='apis_entities.Place'),
        ),
        migrations.AddField(
            model_name='placeplace',
            name='related_placeB',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_placeA', to='apis_entities.Place'),
        ),
        migrations.AddField(
            model_name='placeplace',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='placeplace_set', to='apis_vocabularies.PlacePlaceRelation'),
        ),
        migrations.AddField(
            model_name='placeevent',
            name='related_event',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='placeevent_set', to='apis_entities.Event'),
        ),
        migrations.AddField(
            model_name='placeevent',
            name='related_place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='placeevent_set', to='apis_entities.Place'),
        ),
        migrations.AddField(
            model_name='placeevent',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='placeevent_set', to='apis_vocabularies.PlaceEventRelation'),
        ),
        migrations.AddField(
            model_name='personwork',
            name='related_person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personwork_set', to='apis_entities.Person'),
        ),
        migrations.AddField(
            model_name='personwork',
            name='related_work',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personwork_set', to='apis_entities.Work'),
        ),
        migrations.AddField(
            model_name='personwork',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personwork_set', to='apis_vocabularies.PersonWorkRelation'),
        ),
        migrations.AddField(
            model_name='personplace',
            name='related_person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personplace_set', to='apis_entities.Person'),
        ),
        migrations.AddField(
            model_name='personplace',
            name='related_place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personplace_set', to='apis_entities.Place'),
        ),
        migrations.AddField(
            model_name='personplace',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personplace_set', to='apis_vocabularies.PersonPlaceRelation'),
        ),
        migrations.AddField(
            model_name='personperson',
            name='related_personA',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_personB', to='apis_entities.Person'),
        ),
        migrations.AddField(
            model_name='personperson',
            name='related_personB',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_personA', to='apis_entities.Person'),
        ),
        migrations.AddField(
            model_name='personperson',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personperson_set', to='apis_vocabularies.PersonPersonRelation'),
        ),
        migrations.AddField(
            model_name='personinstitution',
            name='related_institution',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personinstitution_set', to='apis_entities.Institution'),
        ),
        migrations.AddField(
            model_name='personinstitution',
            name='related_person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personinstitution_set', to='apis_entities.Person'),
        ),
        migrations.AddField(
            model_name='personinstitution',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personinstitution_set', to='apis_vocabularies.PersonInstitutionRelation'),
        ),
        migrations.AddField(
            model_name='personevent',
            name='related_event',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personevent_set', to='apis_entities.Event'),
        ),
        migrations.AddField(
            model_name='personevent',
            name='related_person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personevent_set', to='apis_entities.Person'),
        ),
        migrations.AddField(
            model_name='personevent',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personevent_set', to='apis_vocabularies.PersonEventRelation'),
        ),
        migrations.AddField(
            model_name='institutionwork',
            name='related_institution',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionwork_set', to='apis_entities.Institution'),
        ),
        migrations.AddField(
            model_name='institutionwork',
            name='related_work',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionwork_set', to='apis_entities.Work'),
        ),
        migrations.AddField(
            model_name='institutionwork',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionwork_set', to='apis_vocabularies.InstitutionWorkRelation'),
        ),
        migrations.AddField(
            model_name='institutionplace',
            name='related_institution',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionplace_set', to='apis_entities.Institution'),
        ),
        migrations.AddField(
            model_name='institutionplace',
            name='related_place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionplace_set', to='apis_entities.Place'),
        ),
        migrations.AddField(
            model_name='institutionplace',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionplace_set', to='apis_vocabularies.InstitutionPlaceRelation'),
        ),
        migrations.AddField(
            model_name='institutioninstitution',
            name='related_institutionA',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_institutionB', to='apis_entities.Institution'),
        ),
        migrations.AddField(
            model_name='institutioninstitution',
            name='related_institutionB',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_institutionA', to='apis_entities.Institution'),
        ),
        migrations.AddField(
            model_name='institutioninstitution',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutioninstitution_set', to='apis_vocabularies.InstitutionInstitutionRelation'),
        ),
        migrations.AddField(
            model_name='institutionevent',
            name='related_event',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionevent_set', to='apis_entities.Event'),
        ),
        migrations.AddField(
            model_name='institutionevent',
            name='related_institution',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionevent_set', to='apis_entities.Institution'),
        ),
        migrations.AddField(
            model_name='institutionevent',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='institutionevent_set', to='apis_vocabularies.InstitutionEventRelation'),
        ),
        migrations.AddField(
            model_name='eventwork',
            name='related_event',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='eventwork_set', to='apis_entities.Event'),
        ),
        migrations.AddField(
            model_name='eventwork',
            name='related_work',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='eventwork_set', to='apis_entities.Work'),
        ),
        migrations.AddField(
            model_name='eventwork',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='eventwork_set', to='apis_vocabularies.EventWorkRelation'),
        ),
        migrations.AddField(
            model_name='eventevent',
            name='related_eventA',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_eventB', to='apis_entities.Event'),
        ),
        migrations.AddField(
            model_name='eventevent',
            name='related_eventB',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_eventA', to='apis_entities.Event'),
        ),
        migrations.AddField(
            model_name='eventevent',
            name='relation_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='eventevent_set', to='apis_vocabularies.EventEventRelation'),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError

from browsing.models import BrowsConf


class Command(BaseCommand):

    help = "Delete all BrowsConf objects"

    def handle(self, *args, **options):
        brc_objs = BrowsConf.objects.all()
        self.stdout.write("{} objects will be deleted".format(brc_objs.count()))
        brc_objs.delete()
        self.stdout.write("All BrowsConf objects are gone")



from django.conf import settings
from rdflib import RDF, RDFS, XSD, BNode, Literal, URIRef, OWL

try:
    from webpage.metadata import PROJECT_METADATA
except ImportError:
    try:
        from webpage.utils import PROJECT_METADATA
    except ImportError:
        PROJECT_METADATA = getattr(settings, "PROJECT_DEFAULT_MD")

base_uri = getattr(settings, "APIS_BASE_URI", "http://apis.info")
if base_uri.endswith("/"):
    base_uri = base_uri[:-1]
lang = getattr(settings, "LANGUAGE_CODE", "de")


def get_skos_url():
    set_skos = getattr(settings, "APIS_SKOSMOS")
    base_uri = set_skos.get("url")
    if base_uri.endswith("/"):
        base_uri = base_uri[:-1]
    title = PROJECT_METADATA.get("title", "TITLE")
    v_title = set_skos.get("vocabs-name", False)
    if not v_title:
        v_title = f"{title.lower()}thesaurus/"
    uri_1 = f"{base_uri}/{v_title}"
    return uri_1


def m_h_find_map_function(rel, ent_type):
    r1 = [
        (k.split("_")[0], k.split("_")[1], k, cd_mp[k])
        for k in cd_mp.keys()
        if k.startswith(ent_type)
    ]
    res = []
    for rr in r1:
        if "*" not in rr[1]:
            if f"{ent_type}_{rel}" == rr[2]:
                return rr[3]
        elif len(rr[1]) > 1:
            if rr[1].startswith("*") and rel.endswith(rr[1][1:]):
                res.append((1, rr[3]))
            elif rr[1].endswith("*") and rel.startswith(rr[1][:-1]):
                res.append((1, rr[3]))
        else:
            res.append((2, rr[3]))
    if len(res) > 0:
        res.sort(key=lambda tup: tup[0])
        return res[0][1]
    else:
        return None


def m_add_uris(g, ns, obj, uris):
    for u in uris:
        b_uri = URIRef(u["uri"])
        if obj != b_uri:
            g.add((obj, OWL.sameAs, b_uri))
            g.add((b_uri, RDF.type, ns["cidoc"].E42_Identifier))
    return g


def m_add_time_span(g, ns, event, date, date_written, id_ev):
    time_span = URIRef(f"{base_uri}/appellation/{event}/date/{id_ev}")
    g.set((time_span, RDF.type, ns["cidoc"]["E52_Time-Span"]))
    g.set(
        (
            time_span,
            ns["cidoc"].P82a_begin_of_the_begin,
            Literal(date, datatype=XSD.date),
        )
    )
    g.set(
        (time_span, ns["cidoc"].P82b_end_of_the_end, Literal(date, datatype=XSD.date))
    )
    g.set((time_span, RDFS.label, Literal(f"{date_written}")))
    return g, time_span


def m_place_of_birth(g, p, ns, data):
    if (None, ns["cidoc"].P98_brought_into_life, p) in g:
        b_birth = g.value(None, ns["cidoc"].P98_brought_into_life, p, any=False)
    else:
        b_birth = URIRef(f"{base_uri}/entity/{data['id']}/birth")
        """URI for person specific birth event"""
        g.set((b_birth, RDF.type, ns["cidoc"].E67_Birth))
        g.set((b_birth, ns["cidoc"].P98_brought_into_life, p))
    g, place_of_birth = m_place(g, ns, data["target"])
    g.set((b_birth, ns["cidoc"].P7_took_place_at, place_of_birth))

    return g


def m_place_of_death(g, p, ns, data):
    if (None, ns["cidoc"].P100_was_death_of, p) in g:
        b_death = g.value(None, ns["cidoc"].P100_was_death_of, p, any=False)
    else:
        b_death = URIRef(f"{base_uri}/entity/{data['id']}/death")
        """URI for person specific death event"""
        g.set((b_death, RDF.type, ns["cidoc"].E69_Death))
        g.set((b_death, ns["cidoc"].P100_was_death_of, p))
    g, place_of_death = m_place(g, ns, data["target"])
    g.set((b_death, ns["cidoc"].P7_took_place_at, place_of_death))

    return g


def m_institutions_rel(g, p, ns, data):
    b_joining = URIRef(f"{base_uri}/events/joined/{data['id']}")
    g.add((b_joining, RDF.type, ns["cidoc"].E85_Joining))
    g.add((b_joining, ns["cidoc"].P143_joined, p))
    b_pc_joining = URIRef(f"{base_uri}/events/joined_pc/{data['id']}")
    g.add((b_pc_joining, RDF.type, ns["cidoc"].PC144_joined_with))
    g.add((b_pc_joining, ns["cidoc"].P01_has_domain, b_joining))
    inst_uri = URIRef(f"{base_uri}/entity/{data['target']['id']}")
    join_type = URIRef(f"{get_skos_url()}/{data['relation_type']['id']}")
    g.add((join_type, RDF.type, ns["cidoc"].E55_Type))
    g.add((join_type, RDFS.label, Literal(data["relation_type"]["label"])))
    g.add((b_pc_joining, ns["cidoc"]["P144.1_kind_of_member"], join_type))
    if (inst_uri, RDF.type, ns["cidoc"].E74_Group) not in g:
        g, inst_uri = m_institution(g, ns, data["target"])
    if data["start_date"] is not None:
        g, start_date_node = m_add_time_span(
            g, ns, "joined", data["start_date"], data["start_date_written"], data["id"]
        )
        g.set((b_joining, ns["cidoc"]["P4_has_time-span"], start_date_node))
    if data["end_date"] is not None:
        b_leaving = URIRef(f"{base_uri}/events/left/{data['id']}")
        g.add((b_leaving, RDF.type, ns["cidoc"].E86_Leaving))
        g.add((b_leaving, ns["cidoc"].P145_seperated, p))
        g.add((b_leaving, ns["cidoc"].P146_seperated_from, inst_uri))
        g, end_date_node = m_add_time_span(
            g, ns, "left", data["end_date"], data["end_date_written"], data["id"]
        )
        g.set((b_leaving, ns["cidoc"]["P4_has_time-span"], end_date_node))
    g.add((b_pc_joining, ns["cidoc"].P02_has_range, inst_uri))
    return g


cd_mp = {
    "places_place of birth": m_place_of_birth,
    "places_geboren in": m_place_of_birth,
    "places_place of death": m_place_of_death,
    "places_gestorben in": m_place_of_death,
    "institutions_*": m_institutions_rel,
}


def m_place(g, ns, data, drill_down=False):
    place_uri = URIRef("{}/entity/{}".format(base_uri, data["id"]))
    """URI for location"""
    if (place_uri, RDF.type, ns["cidoc"].E53_Place) in g and not drill_down:
        return g, place_uri
    else:
        g.set((place_uri, RDF.type, ns["cidoc"].E53_Place))
        b_place_app1 = URIRef(f"{base_uri}/appellation/place/{data['id']}")
        """URI for location appellation"""
        g.set((place_uri, ns["cidoc"].P1_is_identified_by, b_place_app1))
        g.set((b_place_app1, RDF.type, ns["cidoc"].E41_Appellation))
        g.set(
            (
                b_place_app1,
                RDFS.label,
                Literal(data["name"], lang=lang),
            )
        )
        g.set((place_uri, RDFS.label, Literal(data["name"], lang=lang)))
        if data["lng"] is not None:
            g.set(
                (
                    place_uri,
                    ns["cidoc"].P168_place_is_defined_by,
                    Literal(
                        f"Point( {data['lng']} {data['lat']} )",
                        datatype="geo:wktLiteral",
                    ),
                )
            )
        g = m_add_uris(g, ns, place_uri, data["uris"])
        if drill_down:
            for ent_1 in data["relations"]:
                for p in data["relations"][ent_1]:
                    rel_map = m_h_find_map_function(p["relation_type"]["label"], ent_1)
                    if rel_map is not None:
                        g = rel_map(g, place_uri, ns, p)
        return g, place_uri


def m_work(g, ns, data, drill_down=False):
    k_uri = URIRef(data["url"])
    if (k_uri, RDF.type, ns["cidoc"].E73_Information_Object) in g and not drill_down:
        return g, k_uri
    else:
        k_label_string = f"{data['name']}".strip()
        g.set((k_uri, RDF.type, ns["cidoc"].E73_Information_Object))
        g.set((k_uri, RDFS.label, Literal(k_label_string, lang=lang)))
        b_app = BNode()
        g.set((k_uri, ns["cidoc"].P1_is_identified_by, b_app))
        g.set((b_app, RDF.type, ns["cidoc"].E41_Appellation))
        g.set((b_app, RDFS.label, Literal(k_label_string, lang=lang)))
        g = m_add_uris(g, ns, k_uri, data["uris"])
        return g, k_uri


def m_person(g, ns, data, drill_down=False):
    k_uri = URIRef(data["url"])
    if (k_uri, RDF.type, ns["cidoc"].E21_Person) in g and not drill_down:
        return g, k_uri
    else:
        g.set((k_uri, RDF.type, ns["cidoc"].E21_Person))
        apisid_stmt = URIRef(f"{base_uri}/entity/{data['id']}/apisid")
        """URI for APIS ID"""
        oblid_stmt = URIRef(f"{base_uri}/entity/{data['id']}/oblid")
        """URI for OEBL ID"""
        pnappell_stmt = URIRef(
            f"{base_uri}/pnappellation#{data['id']}/{data['name'].replace(' ', '_')}"
        )
        """URI for preferred name"""
        lname_stmt = URIRef(
            f"{base_uri}/appellationsn#{data['id']}/{data['name'].replace(' ', '_')}"
        )
        """URI for surname"""
        fname_stmt = URIRef(
            f"{base_uri}/appellationfn#{data['id']}/{data['name'].replace(' ', '_')}"
        )
        """URI for last name"""
        gender_stmt = URIRef(f"{get_skos_url()}/gender/{data['gender']}")
        """replace URI with valid acdh.vocabs-URI and use namespace"""
        g.set((pnappell_stmt, ns["cidoc"].P2_has_type, ns["cidoc"].E55_Type))
        g.set((k_uri, ns["cidoc"].P1_is_identified_by, apisid_stmt))
        g.add((k_uri, ns["cidoc"].P1_is_identified_by, pnappell_stmt))
        g.set((apisid_stmt, RDF.type, ns["cidoc"].E42_Identifier))
        g.set(
            (
                apisid_stmt,
                RDFS.label,
                Literal(
                    "is identified by APIS Identifier as {}".format(data["id"]),
                    lang=lang,
                ),
            )
        )
        g.set(
            (
                apisid_stmt,
                RDF.value,
                Literal(data["id"]),
            )
        )
        g.add((k_uri, ns["cidoc"].P1_is_identified_by, oblid_stmt))
        g.set((oblid_stmt, RDF.type, ns["cidoc"].E42_Identifier))
        g.set(
            (
                oblid_stmt,
                RDFS.label,
                Literal("is identified by ÖBL Identifier as XXXX", lang=lang),
                # Literal("is identified by ÖBL Identifier as {}".format(data['orig_id']), lang=lang),
            )
        )
        g.set(
            (
                oblid_stmt,
                RDF.value,
                Literal("XXXX"),
                # Literal(data['orig_id']),
            )
        )
        g.set(
            (
                pnappell_stmt,
                ns["cidoc"]["P2_has_type"],
                URIRef(f"{get_skos_url()}/name/preferrred_name"),
            )
        )
        """replace URI with valid acdh.vocabs-URI and use namespace"""
        g.set(
            (
                pnappell_stmt,
                RDFS.label,
                Literal(
                    f"Preferred name is {data['first_name']} {data['name']}", lang=lang
                ),
            )
        )
        g.set((pnappell_stmt, ns["cidoc"]["P106_is_composed_of"], lname_stmt))
        g.add((pnappell_stmt, ns["cidoc"]["P106_is_composed_of"], fname_stmt))
        g.set(
            (
                lname_stmt,
                ns["cidoc"]["P2_has_type"],
                URIRef(f"{get_skos_url()}/name/surname"),
            )
        )
        """replace with valid acdh.vocabs-URI and use namespace"""
        g.set(
            (lname_stmt, RDFS.label, Literal(f"Surname is {data['name']}", lang=lang))
        )
        g.add((lname_stmt, RDF.value, Literal(data["name"])))
        g.add(
            (
                fname_stmt,
                ns["cidoc"]["P2_has_type"],
                URIRef(f"{get_skos_url()}/name/forename"),
            )
        )
        """replace with valid acdh.vocabs-URI and use namespace"""
        g.set(
            (
                fname_stmt,
                RDFS.label,
                Literal(f"Forename is {data['first_name']}", lang=lang),
            )
        )
        g.add((fname_stmt, RDF.value, Literal(data["first_name"])))
        g.set(
            (
                k_uri,
                RDFS.label,
                Literal(
                    f"Person known as {data['first_name']} {data['name']}", lang=lang
                ),
            )
        )
        # import alternative names: if data["alternate_name"] is not None:
        # g.setusw.
        if data["gender"] != "":
            """Statement for known gender"""
            g.set(((k_uri, ns["cidoc"]["P2_has_type"], gender_stmt)))
            g.add((gender_stmt, RDF.type, ns["cidoc"].E55_Type))
            g.add((gender_stmt, RDF.value, Literal(data["gender"])))
            g.add(
                (
                    gender_stmt,
                    RDFS.label,
                    Literal(
                        "Person was documented as {}".format(data["gender"]), lang=lang
                    ),
                )
            )
        if data["gender"] == "":
            """Statement for unknown gender"""
            g.add(
                (
                    (
                        k_uri,
                        ns["cidoc"]["P2_has_type"],
                        URIRef(f"{get_skos_url()}/gender/unknown"),
                    )
                )
            )
            g.add((gender_stmt, RDF.type, ns["cidoc"].E55_Type))
            g.add((gender_stmt, RDF.value, Literal("unknown")))
            g.add(
                (
                    gender_stmt,
                    RDFS.label,
                    Literal("No Gender was documented.", lang=lang),
                )
            )
        if data["start_date"] is not None:
            if len(data["start_date"]) > 0:
                b_birth = URIRef(f"{base_uri}/entity/{data['id']}/birth")
                """URI for person specific birth event"""
                g.set((b_birth, RDF.type, ns["cidoc"].E67_Birth))
                g.set(
                    (
                        b_birth,
                        RDFS.label,
                        Literal(
                            f"Birth of {data['first_name']} {data['name']}", lang="de"
                        ),
                    )
                )
                b_birth_time_span = URIRef(
                    f"{base_uri}/entity/timestamp/{data['start_date']}"
                )
                g.set((b_birth, ns["cidoc"]["P4_has_time-span"], b_birth_time_span))
                g.set((b_birth_time_span, RDF.type, ns["cidoc"]["E52_Time-Span"]))
                g.set(
                    (
                        b_birth_time_span,
                        ns["cidoc"].P82a_begin_of_the_begin,
                        Literal(data["start_date"], datatype=XSD.date),
                    )
                )
                g.set(
                    (
                        b_birth_time_span,
                        ns["cidoc"].P82b_end_of_the_end,
                        Literal(data["start_date"], datatype=XSD.date),
                    )
                )
                g.set((b_birth_time_span, RDF.value, Literal(f"{data['start_date']}")))
                g.set((b_birth, ns["cidoc"].P98_brought_into_life, k_uri))
        if data["end_date"] is not None:
            if len(data["end_date"]) > 0:
                b_death = URIRef(f"{base_uri}/entity/{data['id']}/death")
                g.set((b_death, RDF.type, ns["cidoc"].E69_Death))
                g.set(
                    (
                        b_death,
                        RDFS.label,
                        Literal(
                            f"Death of {data['first_name']} {data['name']}", lang="de"
                        ),
                    )
                )
                b_death_time_span = URIRef(
                    f"{base_uri}/entity/timestamp/{data['end_date']}"
                )
                g.set((b_death, ns["cidoc"]["P4_has_time-span"], b_death_time_span))
                g.set((b_death_time_span, RDF.type, ns["cidoc"]["E52_Time-Span"]))
                g.set(
                    (
                        b_death_time_span,
                        ns["cidoc"].P82a_begin_of_the_begin,
                        Literal(data["end_date"], datatype=XSD.date),
                    )
                )
                g.set(
                    (
                        b_death_time_span,
                        ns["cidoc"].P82b_end_of_the_end,
                        Literal(data["end_date"], datatype=XSD.date),
                    )
                )
                g.set((b_death_time_span, RDF.value, Literal(f"{data['end_date']}")))
                g.set((b_death, ns["cidoc"].P100_was_death_of, k_uri))
        if data["profession"] is not None:
            for prof in data["profession"]:
                prof_stmt = URIRef(
                    f"{get_skos_url()}/profession/{prof['name'].replace(' ', '_')}"
                )
                g.set((k_uri, ns["cidoc"].P14i_performed, prof_stmt))
                g.set((prof_stmt, RDF.type, ns["frbroo"].F51))
                pref_prof = prof["label"]
                g.set((prof_stmt, RDF.value, Literal(pref_prof)))
                """adress most specific value or profession e.g. Architekt, Schriftsteller, current datatype is List with Ordered Dict"""
        if drill_down:
            for ent_1 in data["relations"]:
                for p in data["relations"][ent_1]:
                    rel_map = m_h_find_map_function(p["relation_type"]["label"], ent_1)
                    if rel_map is not None:
                        g = rel_map(g, k_uri, ns, p)
    g = m_add_uris(g, ns, k_uri, data["uris"])
    return g, k_uri


def m_institution(g, ns, data):
    inst_uri = URIRef(f"{base_uri}/entity/{data['id']}")
    if (inst_uri, RDF.type, ns["cidoc"].E74_Group) in g:
        return g, inst_uri
    else:
        g.set((inst_uri, RDF.type, ns["cidoc"].E74_Group))
        g.set((inst_uri, RDFS.label, Literal(data["name"], lang=lang)))
        if data["kind"] is not None:
            uri_type = URIRef(f"{get_skos_url()}/{data['kind']['id']}")
            g.set((uri_type, RDF.type, ns["cidoc"].E55_Type))
            g.set((inst_uri, ns["cidoc"].P2_has_type, uri_type))
            g.set((uri_type, RDFS.label, Literal(data["kind"]["label"])))
    return g, inst_uri



from django.views.generic import TemplateView

try:
    from webpage.utils import PROJECT_METADATA as PM
except ImportError:
    from django.conf import settings

    PM = getattr(settings, "PROJECT_DEFAULT_MD")
from .models import AboutTheProject, TeamMember


class TeamView(TemplateView):
    template_name = "infos/team.html"

    def get_context_data(self, **kwargs):
        objects = TeamMember.objects.all()
        context = super().get_context_data(**kwargs)
        context["objects"] = objects
        return context


class SpecialAboutView(TemplateView):
    template_name = "infos/about.html"

    def get_context_data(self, **kwargs):
        try:
            object = AboutTheProject.objects.all()[0]
        except IndexError:
            object = PM
        context = super().get_context_data(**kwargs)
        context["object"] = object
        return context



TEI_NSMAP = {
    'tei': "http://www.tei-c.org/ns/1.0",
    'xml': "http://www.w3.org/XML/1998/namespace",
}


PERS_TO_TEI_DICT = {
    'name': "{http://www.tei-c.org/ns/1.0}surname",
    'first_name': "{http://www.tei-c.org/ns/1.0}forename",
    'start_date': "{http://www.tei-c.org/ns/1.0}birth",
    'end_date': "{http://www.tei-c.org/ns/1.0}death",
}


tei_gen_header = """
<TEI xmlns="http://www.tei-c.org/ns/1.0">
    <teiHeader>
        <fileDesc>
         <titleStmt>
            <title type="main">{}</title>
            <title type="sub">{}</title>
         </titleStmt>
         <publicationStmt>
            <p/>
         </publicationStmt>
         <sourceDesc>
            <p></p>
         </sourceDesc>
        </fileDesc>
    </teiHeader>
    <text>
        <group>
            <text type="entity">
                <body/>
            </text>
        </group>
      
    </text>
</TEI>
"""



from django.contrib import admin
from browsing.models import BrowsConf


class BrowsConfAdmin(admin.ModelAdmin):
    list_display = [
        'model_name',
        'label',
        'field_path',
    ]


admin.site.register(BrowsConf, BrowsConfAdmin)



# Generated by Django 2.1.12 on 2020-01-21 12:27

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_vocabularies', '0001_initial'),
        ('auth', '0009_alter_user_last_name_max_length'),
        ('apis_metainfo', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='text',
            name='kind',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.TextType'),
        ),
        migrations.AddField(
            model_name='text',
            name='source',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_metainfo.Source'),
        ),
        migrations.AddField(
            model_name='tempentityclass',
            name='collection',
            field=models.ManyToManyField(to='apis_metainfo.Collection'),
        ),
        migrations.AddField(
            model_name='tempentityclass',
            name='source',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_metainfo.Source'),
        ),
        migrations.AddField(
            model_name='tempentityclass',
            name='text',
            field=models.ManyToManyField(blank=True, to='apis_metainfo.Text'),
        ),
        migrations.AddField(
            model_name='collection',
            name='collection_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.CollectionType'),
        ),
        migrations.AddField(
            model_name='collection',
            name='groups_allowed',
            field=models.ManyToManyField(to='auth.Group'),
        ),
        migrations.AddField(
            model_name='collection',
            name='parent_class',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_metainfo.Collection'),
        ),
    ]



# Generated by Django 2.1.2 on 2020-02-21 17:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_labels', '0002_auto_20200121_1227'),
    ]

    operations = [
        migrations.AddField(
            model_name='label',
            name='end_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='label',
            name='end_date_written',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='End'),
        ),
        migrations.AddField(
            model_name='label',
            name='end_end_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='label',
            name='end_start_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='label',
            name='start_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='label',
            name='start_date_written',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Start'),
        ),
        migrations.AddField(
            model_name='label',
            name='start_end_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='label',
            name='start_start_date',
            field=models.DateField(blank=True, null=True),
        ),
    ]



from rest_framework.generics import ListAPIView
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import filters
from rest_framework.generics import ListAPIView

from apis_core.apis_relations.models import AbstractRelation
from .serializers import *


class GetVisJson(ListAPIView):
    schema = None
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    depth = 2
    # TODO: add a generic filter thing

    def get_serializer(self, instance=None, data=None, many=False, partial=False):
        vis = self.request.query_params.get('vis', None)
        if vis == 'av-age':
            return VisAgeSerializer(self.get_queryset(), many=False)
        elif vis == 'avg-relations':
            return AvRelations(self.get_queryset(), many=False)
        else:
            return None

    def get_queryset(self, **kwargs):
        relation = self.kwargs['relation'].lower()
        relation_model = AbstractRelation.get_relation_class_of_name(relation)
        print("from get_queryset {}".format(relation))
        queryset = relation_model.objects.all()
        return queryset



from django.core.management.base import BaseCommand, CommandError

from browsing.browsing_utils import create_brows_config_obj


class Command(BaseCommand):

    help = "Create BrowsConf objects for app"

    def add_arguments(self, parser):
        parser.add_argument(
            'app_name', type=str,
            help="Name of the app for which you'd like to create BrowsConf objects."
        )

    def handle(self, *args, **kwargs):
        app_name = kwargs['app_name']
        # exclude_fields = kwargs['exclude_fields']
        create_brows_config_obj(app_name)



from django import template
from infos.models import ProjectInst

register = template.Library()


@register.simple_tag
def project_insts():

    """returns all Project Institution objects"""
    project_insts = ProjectInst.objects.all()
    return project_insts



import math
import re
from datetime import datetime, timedelta


def parse_date( date_string: str ) -> (datetime, datetime, datetime):
    """
    function to parse a string date field of an entity

    :param date_string : str :
        the field value passed by a user

    :return date_single : datetime :
        single date which represents either the precise date given by user or median in between a range.

    :return date_ab : datetime :
        starting date of a range if user passed a range value either implicit or explicit.

    :return date_bis : datetime :
        ending date of a range if user passed a range value either implicit or explicit.
    """


    def parse_date_range_individual(date, ab=False, bis=False):
        """
        As a sub function to parse_date, this function parse_date_individual handles a very single date since
        in a text field a user can pass multiple dates.


        :param date : str :
            recognized sub string which potentially is a date (in julian calendar format)

        :param ab : boolean : optional
            indicates if a single date shall be intepreted as a starting date of a range

        :param bis : boolean : optional
            indicates if a single date shall be intepreted as an ending date of a range


        :return tuple (datetime, datetime) :
            two datetime objects representing the dates.
            Two indicate that an implicit single date range was given (e.g. a year without months or days).
            Has to be further processed then since it can be either a starting or ending date range.
        or
        :return datetime :
            One datetime object representing the date.
            if a single date was given.
        """


        def get_last_day_of_month(month, year):
            """
            Helper function to return the last day of a given month and year (respecting leap years)

            :param month : int

            :param year : int

            :return day : int
            """

            if month in [1, 3, 5, 7, 8, 10, 12]:
                # 31 day months
                return 31
            elif month in [4, 6, 9, 11]:
                # 30 day months
                return 30
            elif month == 2:
                # special case february, differentiate leap years with respect to gregorian leap rules
                if year % 4 == 0:
                    if year % 100 == 0:
                        if year % 400 == 0:
                            # divisible by 4, by 100, by 400
                            # thus is leap year
                            return 29
                        else:
                            # divisible by 4, by 100, not by 400
                            # thus is not leap yar
                            return 28
                    else:
                        # divisible by 4, not by 100, if by 400 doesn't matter
                        # thus is leap year
                        return 29
                else:
                    # not divisible by 4, if by 100 or by 400 doesn't matter
                    return 28
            else:
                # no valid month
                raise ValueError("Month " + str(month) + " does not exist.")



        # replace all kinds of delimiters
        date = date.replace(" ", "").replace("-", ".").replace("/", ".").replace("\\", ".")

        # parse into variables for use later
        year = None
        month = None
        day = None

        # check for all kind of Y-M-D combinations
        if re.match(r"\d{3,4}$", date):
            # year
            year = int(date)

        elif re.match(r"\d{1,2}\.\d{3,4}$", date):
            # month - year
            tmp = re.split(r"\.", date)
            month = int(tmp[0])
            year = int(tmp[1])

        elif re.match(r"\d{1,2}\.\d{1,2}\.\d{3,4}$", date):
            # day - month - year
            tmp = re.split(r"\.", date)
            day = int(tmp[0])
            month = int(tmp[1])
            year = int(tmp[2])

        elif re.match(r"\d{3,4}\.\d{1,2}\.?$", date):
            # year - month
            tmp = re.split(r"\.", date)
            year = int(tmp[0])
            month = int(tmp[1])

        elif re.match(r"\d{3,4}\.\d{1,2}\.\d{1,2}\.?$", date):
            # year - month - day
            tmp = re.split(r"\.", date)
            year = int(tmp[0])
            month = int(tmp[1])
            day = int(tmp[2])
        else:
            # No sensical interpretation found
            raise ValueError("Could not interpret date.")


        if (ab and bis) or year is None:
            # both ab and bis in one single date are not valid, neither is the absence of a year.
            raise ValueError("Could not interpret date.")

        elif not ab and not bis and (month is None or day is None):
            # if both ab and bis are False and either month or day is empty, then it was given
            # an implicit date range (range of all months if given a year or all days if given a month)

            # construct implicit month range
            if month is None:
                month_ab = 1
                month_bis = 12
            else:
                month_ab = month
                month_bis = month

            # construct implicit day range
            if day is None:
                day_ab = 1
                day_bis = get_last_day_of_month(month_bis, year)
            else:
                day_ab = day
                day_bis = day


            # return a tuple from a single date (which the calling function has to further process)
            return (
                datetime(year=year, month=month_ab, day=day_ab),
                datetime(year=year, month=month_bis, day=day_bis)
            )

        else:
            # Either ab or bis is True. Then use the respective beginning or end of range and construct a precise date
            # Or both ab and bis are False. Then construct a precise date from parsed values

            # construct implicit month range if month is None
            if month is None:
                if ab and not bis:
                    # is a starting date, thus take first month of year
                    month = 1
                elif not ab and bis:
                    # is an ending date, thus take last month of year
                    month = 12

            # construct implicit day range if day is None
            if day is None:
                if ab and not bis:
                    # is a starting date, thus take first day of month
                    day = 1
                elif not ab and bis:
                    # is an ending date, thus take last month of year
                    day = get_last_day_of_month(month=month, year=year)

            return datetime(year=year, month=month, day=day)


    try:

        # return variables
        date_single = None
        date_ab = None
        date_bis = None

        # split for angle brackets, check if explicit iso date is contained within them
        date_split_angle = re.split(r"(<.*?>)", date_string)


        if len(date_split_angle) > 1:
            # date string contains angle brackets. Parse them, ignore the rest

            def parse_iso_date(date_string):
                date_string_split = date_string.split("-")
                try:
                    return datetime(year=int(date_string_split[0]), month=int(date_string_split[1]), day=int(date_string_split[2]) )
                except:
                    raise ValueError("Invalid iso date: ", date_string)

            if len(date_split_angle) > 3:
                # invalid case
                raise ValueError("Too many angle brackets.")

            elif len(date_split_angle) == 3:
                # the right amount of substrings, indicating exactly one pair of angle brackets.
                # Parse the iso date in between

                # remove angle brackets and split by commas
                dates_iso = date_split_angle[1][1:-1]

                # check for commas, which would indicate that either one iso date or three are being input
                dates_iso = dates_iso.split(",")
                if len(dates_iso) != 1 and len(dates_iso) != 3:
                    # only either one iso date or three are allowed
                    raise ValueError(
                        "Incorrect number of dates given. Within angle brackets only one or three (separated by commas) are allowed.")

                elif len(dates_iso) == 3:
                    # three iso dates indicate further start and end dates

                    # parse start date
                    date_ab_string = dates_iso[1].strip()
                    if date_ab_string != "":
                        date_ab = parse_iso_date(date_ab_string)

                    # parse end date
                    date_bis_string = dates_iso[2].strip()
                    if date_bis_string != "":
                        date_bis = parse_iso_date(date_bis_string)

                # parse single date
                date_single_string = dates_iso[0].strip()
                if date_single_string != "":
                    date_single = parse_iso_date(date_single_string)


        else:
            # date string contains no angle brackets. Interpret the possible date formats
            date_string = date_string.lower()
            date_string = date_string.replace(" ", "")

            # helper variables for the following loop
            found_ab = False
            found_bis = False
            found_single = False

            # split by allowed keywords 'ab' and 'bis' and iterate over them
            date_split_ab_bis = re.split(r"(ab|bis)", date_string)
            for i, v in enumerate(date_split_ab_bis):

                if v == "ab":
                    # indicates that the next value must be a start date

                    if found_ab or found_single:
                        # if already found a ab_date or single date before then there is non-conformative redundancy
                        raise ValueError("Redundant dates found.")
                    found_ab = True

                    # parse the next value which must be a parsable date string
                    date_ab = parse_date_range_individual(date_split_ab_bis[i + 1], ab=True)

                elif v == "bis":
                    # indicates that the next value must be an end date

                    if found_bis or found_single:
                        # if already found a bis_date or single date before then there is non-conformative redundancy
                        raise ValueError("Redundant dates found.")
                    found_bis = True

                    # parse the next value which must be a parsable date string
                    date_bis = parse_date_range_individual(date_split_ab_bis[i + 1], bis=True)

                elif v != "" and not found_ab and not found_bis and not found_single:
                    # indicates that this value must be a date

                    found_single = True

                    # parse the this value which must be a parsable date string
                    date_single = parse_date_range_individual(v)

                    if type(date_single) is tuple:
                        #  if result of parse_date_range_individual is a tuple then the date was an implict range.
                        #  Then split it into start and end dates
                        date_ab = date_single[0]
                        date_bis = date_single[1]

            if date_ab and date_bis:
                # date is a range

                if date_ab > date_bis:
                    raise ValueError("'ab-date' must be before 'bis-date' in time")

                # calculate difference between start and end date of range,
                # and use it to calculate a single date for usage as median.
                days_delta_half = math.floor((date_bis - date_ab).days / 2, )
                date_single = date_ab + timedelta(days=days_delta_half)

            elif date_ab is not None and date_bis is None:
                # date is only the start of a range, save it also as the single date

                date_single = date_ab

            elif date_ab is None and date_bis is not None:
                # date is only the end of a range, save it also as the single date

                date_single = date_bis

    except Exception as e:
        print("Could not parse date: '", date_string, "' due to error: ", e)

    return date_single, date_ab, date_bis



def get_date_help_text_from_dates(single_date, single_start_date, single_end_date, single_date_written):
    """
    function for creating string help text from parsed dates, to provide feedback to the user
    about the parsing status of a given date field.

    :param single_date: datetime :
        the individual date point

    :param single_start_date: datetime :
        the start range of a date

    :param single_end_date: datetime :
        the endrange of a date

    :param single_date_written: str :
        the textual user entry of a date field (needed to check if empty or not)

    :return help_text: str :
        The text to be displayed underneath a date field, informing the user about the parsing result
    """


    # check which of the dates could be parsed to construct the relevant feedback text

    help_text = ""
    if single_date:
        # single date could be parsed

        help_text = "Date interpreted as "

        if single_start_date or single_end_date:
            # date has also start or end ranges, then ignore single date

            if single_start_date:
                # date has start range

                help_text += \
                    str(single_start_date.year) + "-" + \
                    str(single_start_date.month) + "-" + \
                    str(single_start_date.day) + " until "

            else:
                # date has no start range, then write "undefined"

                help_text += "undefined start until "

            if single_end_date:
                # date has end range

                help_text += \
                    str(single_end_date.year) + "-" + \
                    str(single_end_date.month) + "-" + \
                    str(single_end_date.day)

            else:
                # date has no start range, then write "undefined"

                help_text += "undefined end"

        else:
            # date has no start nor end range. Use single date then.

            help_text += \
                str(single_date.year) + "-" + \
                str(single_date.month) + "-" + \
                str(single_date.day)

    elif single_date_written is not None:
        # date input field is not empty but it could not be parsed either. Show parsing info and help text

        help_text = "<b>Date could not be interpreted</b><br>" + get_date_help_text_default()

    else:
        # date field is completely empty. Show help text only

        help_text = get_date_help_text_default()

    return help_text



def get_date_help_text_default():

    return "Dates are interpreted by defined rules. If this fails, an iso-date can be explicitly set with '&lt;YYYY-MM-DD&gt;'."



from rest_framework import serializers

from apis_core.apis_vocabularies.serializers import LabelTypeSerializer
from .models import Label


class LabelSerializerLegacy(serializers.ModelSerializer):
    label_type = LabelTypeSerializer()

    class Meta:
        model = Label
        fields = ('id', 'label', 'isoCode_639_3', 'label_type')



from django.conf.urls import url
from . import views
from . import special_views

app_name = 'infos'
urlpatterns = [
    url(
        r'^project-team/$',
        special_views.TeamView.as_view(),
        name='project-team'
    ),
    url(
        r'^about-the-project/$',
        special_views.SpecialAboutView.as_view(),
        name='about-the-project'
    ),
    url(
        r'^about/$',
        views.AboutTheProjectListView.as_view(),
        name='about_browse'
    ),
    url(
        r'^about/detail/(?P<pk>[0-9]+)$',
        views.AboutTheProjectDetailView.as_view(),
        name='about_detail'
    ),
    url(
        r'^about/create/$',
        views.AboutTheProjectCreate.as_view(),
        name='about_create'
    ),
    url(
        r'^about/edit/(?P<pk>[0-9]+)$',
        views.AboutTheProjectUpdate.as_view(),
        name='about_edit'
    ),
    url(
        r'^about/delete/(?P<pk>[0-9]+)$',
        views.AboutTheProjectDelete.as_view(),
        name='about_delete'),
    url(
        r'^teammember/$',
        views.TeamMemberListView.as_view(),
        name='teammember_browse'
    ),
    url(
        r'^teammember/detail/(?P<pk>[0-9]+)$',
        views.TeamMemberDetailView.as_view(),
        name='teammember_detail'
    ),
    url(
        r'^teammember/create/$',
        views.TeamMemberCreate.as_view(),
        name='teammember_create'
    ),
    url(
        r'^teammember/edit/(?P<pk>[0-9]+)$',
        views.TeamMemberUpdate.as_view(),
        name='teammember_edit'
    ),
    url(
        r'^teammember/delete/(?P<pk>[0-9]+)$',
        views.TeamMemberDelete.as_view(),
        name='teammember_delete'),
    url(
        r'^projectinst/$',
        views.ProjectInstListView.as_view(),
        name='projectinst_browse'
    ),
    url(
        r'^projectinst/detail/(?P<pk>[0-9]+)$',
        views.ProjectInstDetailView.as_view(),
        name='projectinst_detail'
    ),
    url(
        r'^projectinst/create/$',
        views.ProjectInstCreate.as_view(),
        name='projectinst_create'
    ),
    url(
        r'^projectinst/edit/(?P<pk>[0-9]+)$',
        views.ProjectInstUpdate.as_view(),
        name='projectinst_edit'
    ),
    url(
        r'^projectinst/delete/(?P<pk>[0-9]+)$',
        views.ProjectInstDelete.as_view(),
        name='projectinst_delete'),
]



# Generated by Django 2.1.12 on 2020-01-21 12:27

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Collection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Source',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('orig_filename', models.CharField(blank=True, max_length=255)),
                ('indexed', models.BooleanField(default=False)),
                ('pubinfo', models.CharField(blank=True, max_length=400)),
                ('author', models.CharField(blank=True, max_length=255)),
                ('orig_id', models.PositiveIntegerField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='TempEntityClass',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255)),
                ('review', models.BooleanField(default=False, help_text='Should be set to True, if the data record holds up quality standards.')),
                ('start_date', models.DateField(blank=True, null=True)),
                ('start_start_date', models.DateField(blank=True, null=True)),
                ('start_end_date', models.DateField(blank=True, null=True)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('end_start_date', models.DateField(blank=True, null=True)),
                ('end_end_date', models.DateField(blank=True, null=True)),
                ('start_date_written', models.CharField(blank=True, max_length=255, null=True, verbose_name='Start')),
                ('end_date_written', models.CharField(blank=True, max_length=255, null=True, verbose_name='End')),
                ('status', models.CharField(max_length=100)),
                ('references', models.TextField(blank=True, null=True)),
                ('notes', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Text',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Uri',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uri', models.URLField(blank=True, max_length=255, null=True, unique=True)),
                ('domain', models.CharField(blank=True, max_length=255)),
                ('rdf_link', models.URLField(blank=True)),
                ('loaded', models.BooleanField(default=False)),
                ('loaded_time', models.DateTimeField(blank=True, null=True)),
                ('entity', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_metainfo.TempEntityClass')),
            ],
        ),
        migrations.CreateModel(
            name='UriCandidate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uri', models.URLField()),
                ('confidence', models.FloatField(blank=True, null=True)),
                ('responsible', models.CharField(max_length=255)),
                ('entity', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_metainfo.TempEntityClass')),
            ],
        ),
    ]



from django import template
register = template.Library()


@register.inclusion_tag('apis_entities/apis_create_entities.html', takes_context=True)
def apis_create_entities(context):
    values = {}
    return values


@register.simple_tag
def url_replace(request, field, value):
    dict_ = request.GET.copy()
    dict_[field] = value

    return dict_.urlencode()



import json
import os
import pickle
from datetime import date

from django.conf import settings
from django.core.serializers.json import DjangoJSONEncoder
from django.utils import timezone
from rdflib import XSD, Graph, Literal, Namespace, URIRef, ConjunctiveGraph, OWL
from rdflib.namespace import DCTERMS, VOID
from rdflib.plugins.memory import IOMemory
from rdflib.void import generateVoID
from rest_framework import renderers

from apis_core.apis_tei.tei import TeiEntCreator
from .api_mappings.cidoc_mapping import m_person, m_place, m_work, m_institution

try:
    from webpage.metadata import PROJECT_METADATA
except ImportError:
    try:
        from webpage.utils import PROJECT_METADATA
    except ImportError:
        PROJECT_METADATA = getattr(settings, "PROJECT_DEFAULT_MD")


base_uri = getattr(settings, "APIS_BASE_URI", "http://apis.info")
if base_uri.endswith("/"):
    base_uri = base_uri[:-1]
lang = getattr(settings, "LANGUAGE_CODE", "de")


class EntityToTEI(renderers.BaseRenderer):

    media_type = "text/xml"
    format = "tei"

    def render(self, data, media_type=None, renderer_context=None):
        tei_doc = TeiEntCreator(data)
        return tei_doc.serialize_full_doc()


class EntityToCIDOC(renderers.BaseRenderer):

    media_type = "text/rdf"

    ent_func = {
        "Person": m_person,
        "Place": m_place,
        "Work": m_work,
        "Institution": m_institution,
    }

    def render(
        self,
        data1,
        media_type=None,
        renderer_context=None,
        format_1=None,
        binary=False,
        store=False,
        named_graph=None,
        provenance=False,
    ):
        if isinstance(data1, dict):
            data1 = [data1]
        if format_1 is not None:
            self.format = format_1
        cidoc = Namespace("http://www.cidoc-crm.org/cidoc-crm/")
        geo = Namespace("http://www.opengis.net/ont/geosparql#")
        frbroo = Namespace("http://iflastandards.info/ns/fr/frbr/frbroo#")
        if not store:
            store = IOMemory()
        if named_graph:
            uri_entities = URIRef(named_graph)
        else:
            uri_entities = URIRef(f"{base_uri}/entities#")
        g = Graph(store, identifier=uri_entities)
        g.bind("cidoc", cidoc, override=False)
        g.bind("geo", geo, override=False)
        g.bind("owl", OWL, override=False)
        g.bind("frbroo", frbroo, override=False)
        ns = {"cidoc": cidoc, "geo": geo, "frbroo": frbroo}
        if type(data1) == list:
            for data in data1:
                g, ent = self.ent_func[data["entity_type"]](
                    g, ns, data, drill_down=True
                )
        elif type(data1) == str:
            directory = os.fsencode(data1)
            for fn in os.listdir(directory):
                with open(os.path.join(directory, fn), "rb") as inf:
                    data2 = pickle.load(inf)
                    for data in data2:
                        g, ent = self.ent_func[data["entity_type"]](
                            g, ns, data, drill_down=True
                        )
        if provenance:
            g_prov = Graph(
                store, identifier=URIRef("https://omnipot.acdh.oeaw.ac.at/provenance")
            )
            g_prov.bind("dct", DCTERMS, override=False)
            g_prov.bind("void", VOID, override=False)
            g_prov.add(
                (
                    uri_entities,
                    DCTERMS.title,
                    Literal(PROJECT_METADATA["title"], lang=lang),
                )
            )
            g_prov.add(
                (
                    uri_entities,
                    DCTERMS.description,
                    Literal(PROJECT_METADATA["description"], lang=lang),
                )
            )
            g_prov.add(
                (
                    uri_entities,
                    DCTERMS.creator,
                    Literal(PROJECT_METADATA["author"], lang=lang),
                )
            )
            g_prov.add(
                (uri_entities, DCTERMS.publisher, Literal("ACDH-OeAW", lang=lang))
            )
            g_prov.add((uri_entities, DCTERMS.source, URIRef(base_uri)))
            g_prov.add(
                (
                    uri_entities,
                    DCTERMS.created,
                    Literal(str(date.today()), datatype=XSD.date),
                )
            )
            g_prov, g = generateVoID(g, dataset=uri_entities, res=g_prov)
        g_all = ConjunctiveGraph(store=store)
        if binary:
            return g_all, store
        return g_all.serialize(format=self.format.split("+")[-1])


class EntityToCIDOCXML(EntityToCIDOC):

    format = "rdf+xml"


class EntityToCIDOCN3(EntityToCIDOC):

    format = "rdf+n3"


class EntityToCIDOCNQUADS(EntityToCIDOC):

    format = "rdf+nquads"


class EntityToCIDOCTURTLE(EntityToCIDOC):

    format = "rdf+turtle"


class EntityToProsopogrAPhI(renderers.BaseRenderer):

    media_type = "text/json+prosop"
    format = "json+prosop"

    def render(self, data, media_type=None, renderer_context=None):
        factoids = []
        fact_settings = getattr(settings, "PROSOPOGRAPHI_API", None)
        stmt_temp = "Stmt{}_{}"
        f = {"@id": "apis_{}_{}".format(data["entity_type"].lower(), data["id"])}
        f[data["entity_type"].lower()] = {"@id": str(data["id"])}
        f["source"] = {
            "@id": PROJECT_METADATA["title"],
            "label": "{} export".format(PROJECT_METADATA["title"]),
        }
        f["createdBy"] = "{} export".format(PROJECT_METADATA["title"])
        f["createdWhen"] = timezone.now()
        stmt_count = 1
        stmts = []
        f[data["entity_type"].lower()]["uris"] = []
        for u in data["uris"]:
            f[data["entity_type"].lower()]["uris"].append(u["uri"])
        if data["entity_type"].lower() == "person":
            s = {
                "@id": stmt_temp.format(data["id"], stmt_count),
                "name": "{}, {}".format(data["name"], data["first_name"]),
            }
            stmts.append(s)
            stmt_count += 1
        if "end_date" in data.keys():
            if data["end_date"] is not None and data["end_date"] != "":
                s = {
                    "@id": stmt_temp.format(data["id"], stmt_count),
                    "date": {
                        "sortdate": data["end_date"],
                        "label": data["end_date_written"],
                    },
                    "role": {"label": "stirbt"},
                }
                stmts.append(s)
                stmt_count += 1
        if "start_date" in data.keys():
            if data["start_date"] is not None and data["start_date"] != "":
                s = {
                    "@id": stmt_temp.format(data["id"], stmt_count),
                    "date": {
                        "sortdate": data["start_date"],
                        "label": data["start_date_written"],
                    },
                    "role": {"label": "geboren"},
                }
                stmts.append(s)
                stmt_count += 1
        if "gender" in data.keys():
            if data["gender"] is not None and data["gender"] != "":
                s = {
                    "@id": stmt_temp.format(data["id"], stmt_count),
                    "statmentType": [{"label": data["gender"]}],
                    "role": {"uri": "bio-crm:gender", "label": "gender"},
                }
                stmts.append(s)
                stmt_count += 1
        if "profession" in data.keys():
            if len(data["profession"]) > 0:
                s = {
                    "@id": stmt_temp.format(data["id"], stmt_count),
                    "role": {"label": "profession"},
                    "statementType": [],
                }
                for p in data["profession"]:
                    s2 = {
                        "uri": "apis_profession_type:{}".format(p["id"]),
                        "label": p["label"],
                    }
                    s["statementType"].append(s2)
                stmts.append(s)
                stmt_count += 1
        f["statements"] = stmts
        factoids.append(f)
        facts = []
        facts_ind = {}
        if "relations" in data.keys():
            for ent in data["relations"].keys():
                for rel_1 in data["relations"][ent]:
                    s = {
                        "@id": "Stmt{}_rel_{}".format(data["id"], rel_1["id"]),
                        "role": {
                            "label": rel_1["relation_type"]["label"],
                            "url": rel_1["relation_type"]["url"],
                        },
                    }
                    if "start_date" in rel_1.keys():
                        if rel_1["start_date"] != "":
                            s["date"] = {
                                "sortdate": rel_1["start_date"],
                                "label": rel_1["start_date_written"],
                            }
                            if rel_1["end_date_written"] != "":
                                s["date"]["label"] += "-{}".format(
                                    rel_1["end_date_written"]
                                )
                    ext_stc = False
                    t1 = {
                        "uri": "{}/api2/entity/{}".format(
                            base_uri, rel_1["target"]["id"]
                        ),
                        "label": rel_1["target"]["name"],
                    }
                    if fact_settings is not None:
                        if ent in fact_settings.keys():
                            if (
                                rel_1["relation_type"]["label"]
                                in fact_settings[ent].keys()
                            ):
                                s[
                                    fact_settings[ent][rel_1["relation_type"]["label"]]
                                ] = t1
                                ext_stc = True
                    if not ext_stc:
                        s["statementType"] = [t1]
                    try:
                        if len(rel_1["annotation"]) > 0:
                            s["statementContent"] = rel_1["annotation"][0]["text"]
                    except TypeError:
                        pass
                        # stct = {
                        #    "@id": "Annotation_{}".format(rel_1["annotation"][0]["id"]),
                        #    "label": rel_1["annotation"][0]["text"]
                        # }
                        # if "statementContent" in s.keys():
                        #    s["statementContent"].append(stct)
                        # else:
                        #    s["statementContent"] = [stct,]
                    if len(rel_1["revisions"]) > 0:
                        user_1 = rel_1["revisions"][0]["user_created"]
                        date_1 = rel_1["revisions"][0]["date_created"].strftime(
                            "%Y-%m-%d"
                        )
                        rev_id = rel_1["revisions"][0]["id"]
                        if "{}_{}".format(user_1, date_1) not in facts_ind.keys():
                            facts_ind["{}_{}".format(user_1, date_1)] = len(factoids)
                            s3 = {
                                "person": factoids[0]["person"],
                                "@id": "{}_{}".format(factoids[0]["@id"], len(stmts)),
                                "source": {
                                    "@id": "APIS",
                                    "label": "APIS highlighter annotations rev. {}".format(
                                        rev_id
                                    ),
                                },
                                "createdBy": user_1,
                                "createdWhen": date_1,
                                "statements": [s],
                            }
                            factoids.append(s3)
                        else:
                            factoids[facts_ind["{}_{}".format(user_1, date_1)]][
                                "statements"
                            ].append(s)
                            factoids[facts_ind["{}_{}".format(user_1, date_1)]][
                                "source"
                            ]["label"] += " / {}".format(rev_id)
                    else:
                        factoids[0]["statements"].append(s)
        return json.dumps(
            {"factoids": factoids}, sort_keys=True, indent=1, cls=DjangoJSONEncoder
        )



import json

import requests

from apis_core.default_settings.NER_settings import StbGeoQuerySettings, autocomp_settings


def decide_score_stanbol(results, dec_diff):
    if type(results) == dict:
        return results
    if len(results) == 1:
        return results[0]
    res2 = [(r['http://stanbol.apache.org/ontology/entityhub/query#score'][0]['value'], r) for r in results]
    res2.sort(key=lambda tup: tup[0], reverse=True)
    if res2[0][0] > res2[1][0] + dec_diff:
        return res2[0][1]
    else:
        return False


def find_geonames2(ca, name, adm=None, **kwargs):
    headers = {'Content-Type': 'application/json'}
    ca_feature = ca.stored_feature
    if not ca_feature:
        return False
    if adm:
        ca_data = ca.get_data(name, adm)
    else:
        ca_data = ca.get_data(name)
    ca.get_next_feature()
    r = requests.post(ca_feature['URL'], data=json.dumps(ca_data), headers=headers)
    if r.status_code == 200:
        res = r.json()
        if len(res['results']) == 1:
            return True, res['results'][0]
        elif len(res['results']) > 0:
            dec = decide_score_stanbol(res['results'], kwargs['dec_diff'])
            if dec:
                return True, dec
            else:
                return False, res['results']
        else:
            return False, False
    else:
        print(r.content)


def find_loc(lst, geonames_chains=False, dec_diff=5):
    prev_elem = False
    t = False
    if not geonames_chains:
        geonames_chains = []
        for c in autocomp_settings['Place']:
            geonames_chains.append(c['url'])
    if len(lst) == 1:
        pl_selected_fields = StbGeoQuerySettings('place').selected
        headers = {'Content-Type': 'application/json'}
        results = []
        for s in geonames_chains:
            ldpath = ""
            for d in pl_selected_fields:
                ldpath += "{} = <{}>;\n".format(d.split('#')[-1], d)
            data = {
                'limit': 20,
                'name': lst[0],
                'ldpath': ldpath
            }
            r = requests.get(s, params=data, headers=headers)
            if r.status_code == 200:
                res = r.json()
                if len(res['results']) > 0:
                    results.extend(res['results'])
        if len(results) > 1:
            test = decide_score_stanbol(results, dec_diff=dec_diff)
            if test:
                return True, test
            else:
                return False, results
        elif len(results) == 1:
            return True, results
        else:
            return False, False
    elif len(lst) > 1:
        for ind, c in enumerate(lst):
            if ind < len(lst)-1:
                if not t:
                    t = StbGeoQuerySettings('admin')
                if prev_elem:
                    countr = find_geonames2(t, c, prev_elem, dec_diff=dec_diff)
                else:
                    countr = find_geonames2(t, c, dec_diff=dec_diff)
                check = True
                while check:
                    if countr:
                        if countr[0]:
                            if countr[1]['http://www.geonames.org/ontology#featureCode'][0]['value'] == 'http://www.geonames.org/ontology#A.PCLI':
                                prev_elem = (
                                    countr[1]['id'],
                                    'http://www.geonames.org/ontology#parentCountry'
                                )
                            else:
                                prev_elem = (countr[1]['id'], 'http://www.geonames.org/ontology#parent'+countr[1]['http://www.geonames.org/ontology#featureCode'][0]['value'].split('.')[-1])
                            check = False
                        if not countr[0]:
                            check = False
                    else:
                        check = False
            else:
                o = StbGeoQuerySettings('place')
                if prev_elem:
                    place = find_geonames2(o, c, prev_elem, dec_diff=dec_diff)
                else:
                    place = find_geonames2(o, c, dec_diff=dec_diff)
                while place:
                    if place[1]:
                        return place
                    else:
                        if prev_elem:
                            place = find_geonames2(o, c, prev_elem, dec_diff=dec_diff)
                        else:
                            place = find_geonames2(o, c, dec_diff=dec_diff)
                return False


def retrieve_obj(uri):
    headers = {'Content-Type': 'application/json'}
    r = requests.get(
        'http://enrich.acdh.oeaw.ac.at/entityhub/site/geoNames_S_P_A/entity',
        params={'id': uri}, headers=headers
    )
    if r.status_code == 200:
        return r.json()
    else:
        return False


def query_geonames_chains(q, chains=['http://enrich.acdh.oeaw.ac.at/entityhub/site/geoNames_S_P_A/find'],
                          rest_feature=['A', 'P'], dec_diff=5):
    results = []
    ids = []
    headers = {'Content-Type': 'application/json'}
    for chain in chains:
        data = {'limit': 100, 'name': q,
                'ldpath': """name = <http://www.geonames.org/ontology#name>;
                \nfeatureClass = <http://www.geonames.org/ontology#featureClass>;\n
                featureCode = <http://www.geonames.org/ontology#featureCode>;\n"""}
        r = requests.get(chain, params=data, headers=headers)
        res = r.json()
        for t in res['results']:
            if t['id'] not in ids and t['featureClass'][0]['value'].split('#')[1] in rest_feature:
                results.append(t)
                ids.append(t['id'])
    if len(results) > 0:
        dec_st = decide_score_stanbol(results, dec_diff)
        if dec_st:
            return True, dec_st
        else:
            return False, results
    else:
        return False, False



from django import template
from django.contrib.contenttypes.models import ContentType
register = template.Library()


@register.simple_tag
def nav_menu(app=None):

    """creates links to all class of the passed in application
    for which get_listview_url() methods have been registered"""

    if app:
        models = ContentType.objects.filter(app_label=app)
        result = []
        for x in models:
            modelname = x.name
            modelname = modelname.replace(" ", "").lower()
            try:
                fetched_model = ContentType.objects.get(
                    app_label=app, model=modelname).model_class()
                item = {
                    'name': modelname.title(),
                }
            except Exception as e:
                print(e)
                item = {
                    'name': None
                }
            try:
                item['link'] = fetched_model.get_listview_url()
                result.append(item)
            except AttributeError:
                item['link'] = None
        return result


@register.inclusion_tag('browsing/tags/class_definition.html', takes_context=True)
def class_definition(context):
    values = {}
    try:
        values['class_name'] = context['class_name']
        values['docstring'] = context['docstring']
    except Exception as e:
        print(e)
        pass
    return values


@register.inclusion_tag('browsing/tags/column_selector.html', takes_context=True)
def column_selector(context):
    try:
        return {'columns': context['togglable_colums']}
    except Exception as e:
        print(e)
        return {'columns': None}



from django.conf import settings
from django.contrib.auth.mixins import UserPassesTestMixin
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.template.loader import select_template
from django.views import View
from django_tables2 import RequestConfig

from apis_core.apis_labels.models import Label
from apis_core.apis_metainfo.models import Uri
from apis_core.apis_relations.models import AbstractRelation
from apis_core.apis_relations.tables import get_generic_relations_table, LabelTableBase  # , EntityDetailViewLabelTable
from apis_core.helper_functions.utils import access_for_all
from .models import AbstractEntity
from .views import get_highlighted_texts


class GenericEntitiesDetailView(UserPassesTestMixin, View):

    login_url = '/accounts/login/'

    def test_func(self):
        access = access_for_all(self, viewtype="detail")
        return access

    def get(self, request, *args, **kwargs):

        entity = kwargs['entity'].lower()
        pk = kwargs['pk']
        entity_model = AbstractEntity.get_entity_class_of_name(entity)
        instance = get_object_or_404(entity_model, pk=pk)
        relations = AbstractRelation.get_relation_classes_of_entity_name(entity_name=entity)
        side_bar = []
        for rel in relations:
            match = [
                rel.get_related_entity_classA().__name__.lower(),
                rel.get_related_entity_classB().__name__.lower()
            ]
            prefix = "{}{}-".format(match[0].title()[:2], match[1].title()[:2])
            table = get_generic_relations_table(relation_class=rel, entity_instance=instance, detail=True)
            if match[0] == match[1]:
                title_card = entity.title()
                dict_1 = {'related_' + entity.lower() + 'A': instance}
                dict_2 = {'related_' + entity.lower() + 'B': instance}
                if 'apis_highlighter' in settings.INSTALLED_APPS:
                    objects = rel.objects.filter_ann_proj(request=request).filter_for_user().filter(
                        Q(**dict_1) | Q(**dict_2))
                else:
                    objects = rel.objects.filter(
                        Q(**dict_1) | Q(**dict_2))
                    if callable(getattr(objects, 'filter_for_user', None)):
                        objects = objects.filter_for_user()
            else:
                if match[0].lower() == entity.lower():
                    title_card = match[1].title()
                else:
                    title_card = match[0].title()
                dict_1 = {'related_' + entity.lower(): instance}
                if 'apis_highlighter' in settings.INSTALLED_APPS:
                    objects = rel.objects.filter_ann_proj(request=request).filter_for_user().filter(**dict_1)
                else:
                    objects = rel.objects.filter(**dict_1)
                    if callable(getattr(objects, 'filter_for_user', None)):
                        objects = objects.filter_for_user()
            tb_object = table(data=objects, prefix=prefix)
            tb_object_open = request.GET.get(prefix + 'page', None)
            RequestConfig(request, paginate={"per_page": 10}).configure(tb_object)
            side_bar.append(
                (title_card, tb_object, ''.join([x.title() for x in match]), tb_object_open)
            )
        object_lod = Uri.objects.filter(entity=instance)
        object_texts, ann_proj_form = get_highlighted_texts(request, instance)
        object_labels = Label.objects.filter(temp_entity=instance)
        tb_label = LabelTableBase(data=object_labels, prefix=entity.title()[:2]+'L-')
        tb_label_open = request.GET.get('PL-page', None)
        side_bar.append(('Label', tb_label, 'PersonLabel', tb_label_open))
        RequestConfig(request, paginate={"per_page": 10}).configure(tb_label)
        template = select_template([
            'apis_entities/detail_views/{}_detail_generic.html'.format(entity),
            'apis_entities/detail_views/entity_detail_generic.html'
            ])
        tei = getattr(settings, "APIS_TEI_TEXTS", [])
        if tei:
            tei = set(tei) & set([x.kind.name for x in instance.text.all()])
        ceteicean_css = getattr(settings, "APIS_CETEICEAN_CSS", None)
        ceteicean_js = getattr(settings, "APIS_CETEICEAN_JS", None)
        openseadragon_js = getattr(settings, "APIS_OSD_JS", None)
        openseadragon_img = getattr(settings, "APIS_OSD_IMG_PREFIX", None)
        iiif_field = getattr(settings, "APIS_IIIF_WORK_KIND", None)
        if iiif_field:
            try:
                if "{}".format(instance.kind) == "{}".format(iiif_field):
                    iiif = True
                else:
                    iiif = False
            except AttributeError:
                iiif = False
        else:
            iiif = False
        iiif_server = getattr(settings, "APIS_IIIF_SERVER", None)
        iiif_info_json = instance.name
        try:
            no_merge_labels = [
                x for x in object_labels if not x.label_type.name.startswith('Legacy')
            ]
        except AttributeError:
            no_merge_labels = []
        return HttpResponse(template.render(
            request=request, context={
                'entity_type': entity,
                'object': instance,
                'right_card': side_bar,
                'no_merge_labels': no_merge_labels,
                'object_lables': object_labels,
                'object_texts': object_texts,
                'object_lod': object_lod,
                'tei': tei,
                'ceteicean_css': ceteicean_css,
                'ceteicean_js': ceteicean_js,
                'iiif': iiif,
                'openseadragon_js': openseadragon_js,
                'openseadragon_img': openseadragon_img,
                'iiif_field': iiif_field,
                'iiif_info_json': iiif_info_json,
                'iiif_server': iiif_server,
                }
            ))


# TODO __sresch__ : This seems unused. Remove it once sure
# class WorkDetailView(DetailView):
#     model = Work
#     template_name = 'apis_entities/detail_views/work_detail.html'
#
#     def get_context_data(self, **kwargs):
#         context = super(WorkDetailView, self).get_context_data(**kwargs)
#         return context



from django.template.loader import render_to_string
from rest_framework import renderers
from rest_framework_csv.renderers import CSVRenderer


class TEIBaseRenderer(renderers.BaseRenderer):
    media_type = 'application/xml+tei'
    format = 'xml'

    def render(self, data, media_type=None, renderer_context=None):
        data = render_to_string("apis_metainfo/TEI_renderer.xml", {'data': data, 'renderer_context': renderer_context})

        return data


class PaginatedCSVRenderer(CSVRenderer):
    results_field = 'results'

    def render(self, data, *args, **kwargs):
        if not isinstance(data, list):
            data = data.get(self.results_field, [])
        return super(PaginatedCSVRenderer, self).render(data, *args, **kwargs)



from dal import autocomplete

from .models import TempEntityClass


class TempEntityClassAC(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = TempEntityClass.objects.all()

        if self.q:
            qs = qs.filter(name__icontains=self.q)

        return qs



from rest_framework import viewsets

from .models import Collection, Text, Source, Uri, TempEntityClass
from .serializers import (
    CollectionSerializer, TextSerializer, SourceSerializer,
    UriSerializer, TempEntityClassSerializer)


class CollectionViewSet(viewsets.ModelViewSet):
    queryset = Collection.objects.all()
    serializer_class = CollectionSerializer


class TextViewSet(viewsets.ModelViewSet):
    queryset = Text.objects.all()
    serializer_class = TextSerializer


class SourceSerializerViewSet(viewsets.ModelViewSet):
    queryset = Source.objects.all()
    serializer_class = SourceSerializer


class UriSerializerViewSet(viewsets.ModelViewSet):
    queryset = Uri.objects.all()
    serializer_class = UriSerializer


class TempEntityClassViewSet(viewsets.ModelViewSet):
    queryset = TempEntityClass.objects.all()
    serializer_class = TempEntityClassSerializer

    def destroy(self, request, pk=None):
        pass



from django.apps import apps
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    # Show this when the user types help
    help = "creates 'related' vocabs entries for all Relation classes"

    def handle(self, *args, **kwargs):
        for x in apps.get_app_config('apis_vocabularies').get_models():
            if x.__name__.endswith('Relation'):
                x.objects.get_or_create(
                    name="related",
                    name_reverse="related"
                )



import os

import yaml
from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import IntegrityError

from apis_core.apis_metainfo.models import Uri
from apis_core.helper_functions.RDFParser import clean_uri

uri_settings_file = os.path.join(
    settings.BASE_DIR, 'apis_core', 'default_settings', "URI_replace_settings.yml"
)

sett = yaml.load(open(uri_settings_file, 'r'))


class Command(BaseCommand):
    # Show this when the user types help
    help = "normalizes Uris"

    def handle(self, *args, **options):
        for x in Uri.objects.all():
            old_uri = x.uri
            new_uri = clean_uri(sett, old_uri)
            try:
                x.uri = new_uri
                x.save()
            except IntegrityError:
                print(x)
        return "all done"



import pandas as pd
from django.conf import settings
from django.core.management.base import BaseCommand

cwd = settings.BASE_DIR


def create_vocab_item(vocab_class, row, row_key):
    """gets or create a vocab entry based on name and name_reverse"""
    try:
        name_reverse = row[row_key].split("|")[1]
        name = row[row_key].split("|")[0]
    except IndexError:
        name_reverse = row[row_key]
        name = row[row_key]
    temp_item, _ = vocab_class.objects.get_or_create(
        name=name, name_reverse=name_reverse
    )
    return temp_item


class Command(BaseCommand):
    # Show this when the user types help
    help = "creates vocab entries (for relations) stored in the passed in Excel-Sheet"

    def add_arguments(self, parser):
        parser.add_argument('data', type=str, help='Location of your Excel-Sheet')

    # A command must define handle()
    def handle(self, *args, **kwargs):
        file = kwargs['data']
        excel_book = pd.read_excel(file, None)
        for x in excel_book.keys():
            df = pd.read_excel(file, x)
            col_len = len(df.columns)
            for i, row in df.iterrows():
                c = 1
                vocab_class = eval(x)
                while c <= col_len:
                    row_key = "ebene_{}".format(c)
                    if isinstance(row[row_key], str):
                        temp_item = create_vocab_item(vocab_class, row, row_key)
                        if c > 1:
                            parent_key = "ebene_{}".format(c-1)
                            parent = create_vocab_item(vocab_class, row, parent_key)
                            temp_item.parent_class = parent
                            temp_item.save()
                            self.stdout.write("{} crated/updated".format(temp_item))
                    c += 1



from django.conf.urls import url

from . import dal_views

app_name = 'apis_metainfo'

urlpatterns = [
    url(
        r'^tempentity-autocomplete/$', dal_views.TempEntityClassAC.as_view(),
        name='apis_tempentity-autocomplete',
    )
]



# Generated by Django 2.1.12 on 2019-09-30 11:47

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AboutTheProject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=300, verbose_name="Project's Title")),
                ('subtitle', models.CharField(blank=True, max_length=300, verbose_name="Project's Sub Title")),
                ('description', models.TextField(blank=True, verbose_name='Project Description')),
                ('author', models.CharField(blank=True, help_text='The names of the Agents responsible for this description', max_length=250, verbose_name='Authors')),
                ('github', models.CharField(blank=True, help_text="Link to the application's source code", max_length=250, verbose_name='Code Repo')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'About the Project',
                'ordering': ['id'],
            },
        ),
        migrations.CreateModel(
            name='ProjectInst',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=300, verbose_name='Name')),
                ('abbr', models.CharField(blank=True, max_length=300, verbose_name='Abbreviation')),
                ('description', models.TextField(blank=True, verbose_name='Short description of the Institution')),
                ('website', models.URLField(blank=True, max_length=300, verbose_name="Link to the Institution's website")),
                ('logo_url', models.URLField(blank=True, max_length=300, verbose_name="Link to the Insitution's Logo")),
                ('norm_url', models.URLField(blank=True, help_text='URL to any normdata record of the institution', max_length=300, verbose_name='Norm Data URL (OCRID, GND, VIAF, ...)')),
            ],
            options={
                'verbose_name': 'Institution involved in the Project',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='TeamMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=300, verbose_name='Name')),
                ('description', models.TextField(blank=True, verbose_name='Short description of the Person')),
                ('website', models.URLField(blank=True, max_length=300, verbose_name="Link to the person's website")),
                ('role', models.CharField(blank=True, help_text='will be used to group the team member', max_length=300, verbose_name="The person's role in the project")),
                ('norm_url', models.URLField(blank=True, help_text='URL to any normdata record of the person', max_length=300, verbose_name='Norm Data URL (OCRID, GND, VIAF, ...)')),
            ],
            options={
                'verbose_name': 'Team Member',
                'ordering': ['role', 'name'],
            },
        ),
    ]



import django_tables2 as tables
from django_tables2.utils import A
from . models import AboutTheProject, TeamMember, ProjectInst


class AboutTheProjectTable(tables.Table):

    id = tables.LinkColumn(verbose_name='ID')

    class Meta:
        model = AboutTheProject
        sequence = ('id',)
        attrs = {"class": "table table-responsive table-hover"}


class TeamMemberTable(tables.Table):

    id = tables.LinkColumn(verbose_name='ID')

    class Meta:
        model = TeamMember
        sequence = ('id',)
        attrs = {"class": "table table-responsive table-hover"}


class ProjectInstTable(tables.Table):

    id = tables.LinkColumn(verbose_name='ID')

    class Meta:
        model = ProjectInst
        sequence = ('id',)
        attrs = {"class": "table table-responsive table-hover"}



#!/usr/bin/python
# -*- coding: utf-8 -*-

from crispy_forms.helper import FormHelper
from django import forms

from apis_core.apis_entities.models import Place
from apis_core.apis_labels.models import Label
from apis_core.helper_functions import DateParser
from apis_core.helper_functions.RDFParser import RDFParser


##############################################
# Generic
##############################################

class EntityLabelForm(forms.ModelForm):

    class Meta:
        model = Label
        fields = ['label', 'isoCode_639_3', 'label_type', 'start_date_written', 'end_date_written']

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = Label.objects.get(pk=instance)
            x.label = cd['label']
            x.isoCode_639_3 = cd['isoCode_639_3']
            x.label_type = cd['label_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
        else:
            x = super(EntityLabelForm, self).save(commit=False)
            x.temp_entity = site_instance
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(EntityLabelForm, self).__init__(*args, **kwargs)
        self.fields['label'].required = True
        self.fields['label_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'EntityLabelForm'
        self.helper.form_tag = False

        instance = getattr(self, 'instance', None)
        if instance != None:

            if instance.start_date_written:
                self.fields['start_date_written'].help_text = DateParser.get_date_help_text_from_dates(
                    single_date=instance.start_date,
                    single_start_date=instance.start_start_date,
                    single_end_date=instance.start_end_date,
                    single_date_written=instance.start_date_written,
                )
            else:
                self.fields['start_date_written'].help_text = DateParser.get_date_help_text_default()

            if instance.end_date_written:
                self.fields['end_date_written'].help_text = DateParser.get_date_help_text_from_dates(
                    single_date=instance.end_date,
                    single_start_date=instance.end_start_date,
                    single_end_date=instance.end_end_date,
                    single_date_written=instance.end_date_written,
                )
            else:
                self.fields['end_date_written'].help_text = DateParser.get_date_help_text_default()

##############################################
# Person
##############################################


class PersonLabelForm(EntityLabelForm):
    pass

##############################################
# Institutions
##############################################


class InstitutionLabelForm(EntityLabelForm):
    pass


##############################################
# Places
##############################################

class PlaceLabelForm(EntityLabelForm):
    pass


##############################################
# Events
##############################################


class EventLabelForm(EntityLabelForm):
    pass


##############################################
# Entities Base Forms
#############################################


class PlaceEntityForm(forms.Form):
    # place = forms.CharField(label='Place', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        pl = Place.get_or_create_uri(cd['place_uri'])
        if not pl:
            pl = RDFParser(cd['place_uri'], 'Place').get_or_create()
        return pl



from django import template
from django.contrib.contenttypes.models import ContentType

register = template.Library()

@register.filter
def content_type(obj):
    if not obj:
        return False
    if type(obj) == dict:
        for x in obj.keys():
            if x.startswith('related_'):
                nl = [x.split('_')[1], x.split('_')[1]]
    else:
        nl = ContentType.objects.get_for_model(obj).name.split()
    return ''.join([x.title() for x in nl])



from django import template
from django.contrib.contenttypes.models import ContentType

register = template.Library()


@register.simple_tag
def create_object_count(app=None):

    """fetches all models of the passed in app and returns a
    dict containg the name of each class and the number of instances"""

    if app:
        models = ContentType.objects.filter(app_label=app)
        result = []
        for x in models:
            modelname = x.name
            modelname = modelname.replace(" ", "").lower()
            try:
                fetched_model = ContentType.objects.get(
                    app_label=app, model=modelname).model_class()
                item = {
                    'name': modelname.title(),
                    'count': fetched_model.objects.count()
                }
            except:
                item = {
                    'name': x,
                    'count': "Some error occured"
                }
            try:
                item['link'] = fetched_model.get_listview_url()
            except AttributeError:
                item['link'] = None
            result.append(item)
        return result

    else:
        result = [
            {
                'name': 'no parameter passed in',
                'count': '1'
            }
        ]
        return result



import ast
import logging

import reversion
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand, CommandError
from entities.models import Person
from helper_functions.abbreviation_solver import ResolveAbbreviations
from metainfo.models import Text
from vocabularies.models import TextType


class Command(BaseCommand):
    help = 'Command to resolve abbreviations in APIS biographies.'

    def add_arguments(self, parser):
        parser.add_argument(
            '--person',
            action='store',
            dest='person',
            default=False,
            help='Give a dictionary of query arguments for the persons.',
        )

        parser.add_argument(
            '--includeall',
            action='store_true',
            dest='includeall',
            default=False,
            help='Use "includeaall" to include all persons (overides the person argument).',
        )

        parser.add_argument(
            '--texts',
            action='append',
            dest='texts',
            default=[],
            help='Use to give texttypes to resolve.',
        )

        parser.add_argument(
            '--alltexts',
            action='store_true',
            dest='alltexts',
            default=False,
            help='Use to give process all texts associated with the specified person (overrides texts attribute).',
        )

        parser.add_argument(
            '--appendtexttype',
            action='store',
            dest='appendtexttype',
            default=False,
            help='Term to append to the actual texttype\
             to store the resolved text. Original texttypes are used if not given.',
        )

        parser.add_argument(
            '--username',
            action='store',
            dest='username',
            default=False,
            help='Sketch engine username.',
        )

        parser.add_argument(
            '--password',
            action='store',
            dest='password',
            default=False,
            help='Sketch engine password.',
        )

    def handle(self, *args, **options):
        #print(options)
        user = User.objects.get(username='management_bot')
        user_sketch = options['username']
        password_sketch = options['password']
        txt_type_new = False
        if options['includeall']:
            pq = Person.objects.all()
        else:
            if options['person']:
                q_dict = ast.literal_eval(options['person'].strip())
                pq = Person.objects.filter(**q_dict)
            else:
                raise CommandError('You need to specify either a queryset for Persons or use --includeall')
        if options['alltexts']:
            tq = Text.objects.filter(tempentityclass__in=pq)
        else:
            if options['texts']:
                t_type = TextType.objects.filter(name__in=options['texts'])
                tq = Text.objects.filter(tempentityclass__in=pq, kind=t_type)
            else:
                tq = Text.objects.filter(tempentityclass__in=pq)
        with reversion.create_revision():
            for txt in tq[:2]:
                if options['appendtexttype']:
                    txt_type_new, created = TextType.objects.get_or_create(
                        name='{}_{}'.format(options['text'], options['appendtexttype']))
                if not txt_type_new:
                    txt_type_new, created = TextType.objects.get_or_create(name='default type resolved text')
                pers = txt.tempentityclass_set.all().values_list('pk', flat=True)
                pers = Person.objects.filter(pk__in=pers)
                if len(pers) != 1:
                    logging.warning('Could not identify a persons name. Resolving without person names.')
                    pers_name = None
                else:
                    pers_name = [pers[0].first_name, pers[0].name]
                res = ResolveAbbreviations(text=txt.text, person=pers_name, user=user_sketch, password=password_sketch)
                txt_resolved = res.resolve()
                t1 = Text.objects.create(text=txt_resolved, kind=txt_type_new)
                for pers2 in pers:
                    pers2.text.add(t1)



import inspect
import sys
import yaml

# from reversion import revisions as reversion
import reversion
from crum import get_current_request
from django.conf import settings
from django.db import models
from django.db.models import Q

from apis_core.apis_entities.models import Person
from apis_core.apis_metainfo.models import TempEntityClass


#######################################################################
#
# Custom Managers
#
#######################################################################

def find_if_user_accepted():
    request = get_current_request()
    if request is not None:
        print('running through request')
        if request.user.is_authenticated:
            print('authenticated')
            return {}
        else:
            return {'published': True}
    else:
        return {}


class RelationPublishedQueryset(models.QuerySet):
    def filter_for_user(self, *args, **kwargs):
        if getattr(settings, "APIS_SHOW_ONLY_PUBLISHED", False):
            request = get_current_request()
            if request is not None:
                if request.user.is_authenticated:
                    return self
                else:
                    return self.filter(published=True)
            else:
                return self.filter(published=True)
        else:
            return self

    def filter_ann_proj(self, request=None, ann_proj=1, include_all=True):
        """The filter function provided by the manager class.

        :param request: `django.request` object
        :return: queryset that contains only objects that are shown in the highlighted text or those not connected
            to an annotation at all.
        """
        qs = self
        users_show = None
        if request:
            ann_proj = request.session.get('annotation_project', False)
            if not ann_proj:
                return qs
            users_show = request.session.get('users_show_highlighter', None)
        query = Q(annotation__annotation_project_id=ann_proj)
        if users_show is not None:
            query.add(Q(annotation__user_added_id__in=users_show), Q.AND)
        if include_all:
            query.add(Q(annotation__annotation_project__isnull=True), Q.OR)
        return qs.filter(query)


class BaseRelationManager(models.Manager):
    def get_queryset(self):
        return RelationPublishedQueryset(self.model, using=self._db)

    def filter_ann_proj(self, request=None, ann_proj=1, include_all=True):
        return self.get_queryset().filter_ann_proj(request=request, ann_proj=ann_proj, include_all=include_all)

    def filter_for_user(self):
        if hasattr(settings, "APIS_SHOW_ONLY_PUBLISHED") or "apis_highlighter" in getattr(settings, "INSTALLED_APPS"):
            return self.get_queryset().filter_for_user()
        else:
            return self.get_queryset()

#######################################################################
#
# AbstractRelation
#
#######################################################################


class AbstractRelation(TempEntityClass):
    """
    Abstract super class which encapsulates common logic between the different relations and provides various methods
    relating to either all or specific relations.
    """
    objects = BaseRelationManager()

    #annotation_links = AnnotationRelationLinkManager()

    class Meta:
        abstract = True
        default_manager_name = 'objects'



    # Methods dealing with individual data retrievals of instances
    ####################################################################################################################

    def __str__(self):
        return "{} ({}) {}".format(self.get_related_entity_instanceA(), self.relation_type, self.get_related_entity_instanceB())


    def get_web_object(self):

        nameA = self.get_related_entity_instanceA().name
        nameB = self.get_related_entity_instanceB().name

        if self.get_related_entity_classA() == Person:
            nameA += ", "
            if self.get_related_entity_instanceA().first_name is None:
                nameA += "-"
            else:
                nameA += self.get_related_entity_instanceA().first_name

        if self.get_related_entity_classB() == Person:
            nameB += ", "
            if self.get_related_entity_instanceB().first_name is None:
                nameB += "-"
            else:
                nameB += self.get_related_entity_instanceB().first_name

        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            self.get_related_entity_field_nameA(): nameA,
            self.get_related_entity_field_nameB(): nameB,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


    def get_table_dict(self, entity):
        """Dict for the tabels in the html view

        :param entity: Object of type :class:`entities.models.Place`; Used to determine which Place is the main antity
            and which one the related.
        :return:
        """
        if self.get_related_entity_instanceA() == entity:
            rel_other_key = self.get_related_entity_field_nameB()[:-1]
            rel_other_value = self.get_related_entity_instanceB()
            rel_type = self.relation_type.label
        elif self.get_related_entity_instanceB() == entity:
            rel_other_key = self.get_related_entity_field_nameA()[:-1]
            rel_other_value = self.get_related_entity_instanceA()
            rel_type = self.relation_type.label_reverse
        else:
            raise Exception("Did not find corresponding entity. Wiring of current relation to current entity is faulty.")

        result = {
            'relation_pk': self.pk,
            'relation_type': rel_type,
            rel_other_key: rel_other_value,
            'start_date_written': self.start_date_written,
            'end_date_written': self.end_date_written,
            'start_date': self.start_date,
            'end_date': self.end_date}
        return result



    # Various Methods enabling convenient shortcuts between entities, relations, fields, etc
    ####################################################################################################################


    # Methods dealing with all relations
    ####################################################################################################################


    _all_relation_classes = None
    _all_relation_names = None


    @classmethod
    def get_all_relation_classes(cls):
        """
        Instantiates both lists: '_all_relation_classes' and '_all_relation_names'

        :return: list of all python classes of the relations defined within this models' module
        """

        # check if not yet instantiated
        if cls._all_relation_classes == None:
            # if not, then instantiate the private lists

            relation_classes = []
            relation_names = []

            # using python's reflective logic, the following loop iterates over all classes of this current module.
            for relation_name, relation_class in inspect.getmembers(
                    sys.modules[__name__], inspect.isclass):
                print('inspecting classes')
                # check for python classes not to be used.
                if \
                        relation_class.__module__ == "apis_core.apis_relations.models" and \
                        relation_class.__name__ != "AnnotationRelationLinkManager" and \
                        relation_class.__name__ != "BaseRelationManager" and \
                        relation_class.__name__ != "RelationPublishedQueryset" and \
                        relation_class.__name__ != "AbstractRelation" and \
                        relation_name != "ent_class":

                    relation_classes.append(relation_class)
                    relation_names.append(relation_name.lower())

            cls._all_relation_classes = relation_classes
            cls._all_relation_names = relation_names

        return cls._all_relation_classes


    @classmethod
    def get_relation_class_of_name(cls, relation_name):
        """
        :param entity_name: str : The name of an relation
        :return: The model class of the relation respective to the given name
        """

        for relation_class in cls.get_all_relation_classes():
            if relation_class.__name__.lower() == relation_name.lower():
                return relation_class

        raise Exception("Could not find relation class of name:", relation_name)


    @classmethod
    def get_all_relation_names(cls):
        """
        :return: list of all class names in lower case of the relations defined within this models' module
        """

        if cls._all_relation_classes == None:

            # The instantion logic of relation_names list is coupled to the instantiation logic of the relation_classes
            # list done in the method 'get_all_relation_classes'; hence just calling that is sufficient.
            cls.get_all_relation_classes()

        return cls._all_relation_names



    # Methods dealing with related relations and entities
    ####################################################################################################################


    _relation_classes_of_entity_class = {}
    _relation_classes_of_entity_name = {}
    _relation_field_names_of_entity_class = {}


    @classmethod
    def get_relation_classes_of_entity_class(cls, entity_class):
        """
        :param entity_class : class of an entity for which the related relations should be returned
        :return: a list of relation classes that are related to the entity class

        E.g. AbstractRelation.get_relation_classes_of_entity_class( Person )
        -> [ PersonEvent, PersonInstitution, PersonPerson, PersonPlace, PersonWork ]
        """

        return cls._relation_classes_of_entity_class[entity_class]


    @classmethod
    def get_relation_classes_of_entity_name(cls, entity_name):
        """
        :param entity_name : class name of an entity for which the related relations should be returned
        :return: a list of relation classes that are related to the entity class

        E.g. AbstractRelation.get_relation_classes_of_entity_class( 'person' )
        -> [ PersonEvent, PersonInstitution, PersonPerson, PersonPlace, PersonWork ]
        """

        return cls._relation_classes_of_entity_name[entity_name.lower()]


    @classmethod
    def add_relation_class_of_entity_class(cls, entity_class):
        """
        Adds the given entity class to a list which is later retrieved via a dictionary, where the entity class and entity class name
        define the key and the list of related relation classes as their values.

        :param entity_class: the class for which the related relation (the current cls) will be saved into a respective list.
        :return: None
        """

        # get the list of the class dictionary, create if not yet exists.
        relation_class_list = cls._relation_classes_of_entity_class.get(entity_class, [])

        # append the current relation class to the list.
        relation_class_list.append(cls)

        # save into the dictionary, which uses the entity class as key and the extended list above as value.
        cls._relation_classes_of_entity_class[entity_class] = relation_class_list
        cls._relation_classes_of_entity_name[entity_class.__name__.lower()] = relation_class_list


    @classmethod
    def get_relation_field_names_of_entity_class(cls, entity_class):
        """
        :param entity_class : class of an entity for which the related relation class field names should be returned
        :return: a list of relation class field names that are related to the entity class

        E.g. AbstractRelation.get_relation_names_of_entity_class( Person )
        -> [ personevent_set, personinstitution_set, related_personA, related_personB, personplace_set, personwork_set ]
        """

        return cls._relation_field_names_of_entity_class[entity_class]


    @classmethod
    def add_relation_field_name_of_entity_class(cls, relation_name, entity_class):
        """
        Adds the given entity class to a list which is later retrieved via a dictionary, where the entity class
        defines the key and the list of related relation classes as its values.

        :param entity_class: the class for which the related relation (the current cls) will be saved into a respective list.
        :return: None
        """

        # get the list of the class dictionary, create if not yet exists.
        relation_names_list = cls._relation_field_names_of_entity_class.get(entity_class, [])
        # append the current relation field name to the list.
        if relation_name not in relation_names_list: 
            relation_names_list.append(relation_name) #TODO: this is a workaround, find out why it is called several times
        # save into the dictionary, which uses the entity class as key and the extended list above as value.
        cls._relation_field_names_of_entity_class[entity_class] = relation_names_list


    def get_related_entity_instanceA(self):
        """
        This method only works on the relation instance of a given relation class.
        There it returns the instance of an entity on the 'A' side of the given relation instance.

        Note that if your IDE complains about expecting a 'str' instead of 'None' this happens because
        the method 'get_related_entity_field_nameA()' is only implemented and overridden at runtime in the
        function 'generate_all_fields' in the class 'EntityRelationFieldGenerator'.

        :return: An entity instance related to the current relation instance
        """
        return getattr( self, self.get_related_entity_field_nameA() )


    def get_related_entity_instanceB(self):
        """
        This method only works on the relation instance of a given relation class.
        There it returns the instance of an entity on the 'B' side of the given relation instance.

        Note that if your IDE complains about expecting a 'str' instead of 'None' this happens because
        the method 'get_related_entity_field_nameB()' is only implemented and overridden at runtime in the
        function 'generate_all_fields' in the class 'EntityRelationFieldGenerator'.

        :return: An entity instance related to the current relation instance
        """
        return getattr( self, self.get_related_entity_field_nameB() )



    # method stumps
    ####################################################################################################################
    # These stumps merely serve as placeholders so that both IDE and developers know that these methods exist.
    # They are implemented programmatically in the function 'generate_all_fields' in the class 'EntityRelationFieldGenerator'.


    @classmethod
    def get_related_entity_classA(cls):
        """
        :return: the python class of the A side of the current relation class or instance
        E.g. PersonWork -> Person
        """
        return None

    @classmethod
    def get_related_entity_classB(cls):
        """
        :return: the python class of the B side of the current relation class or instance
        E.g. PersonWork -> Work
        """
        return None

    @classmethod
    def get_related_entity_field_nameA(cls):
        """
        :return: the name of the field of the A side of the current relation class or instance
        E.g. PersonWork -> "related_person"
        """
        return None

    @classmethod
    def get_related_entity_field_nameB(cls):
        """
        :return: the name of the field of the B side of the current relation class or instance
        E.g. PersonWork -> "related_work"
        """
        return None




#######################################################################
#
# Person - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class PersonPerson(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class PersonPlace(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class PersonInstitution(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class PersonEvent(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class PersonWork(AbstractRelation):

    pass


#######################################################################
#
#   Institution - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionInstitution(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionPlace(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionEvent(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionWork(AbstractRelation):

    pass


#######################################################################
#
#   Place - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class PlacePlace(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class PlaceEvent(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class PlaceWork(AbstractRelation):

    pass


#######################################################################
#
#   Event - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class EventEvent(AbstractRelation):

    pass


@reversion.register(follow=['tempentityclass_ptr'])
class EventWork(AbstractRelation):

    pass


#######################################################################
#
#   Work - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class WorkWork(AbstractRelation):

    pass


a_ents = getattr(settings, 'APIS_ADDITIONAL_ENTITIES', False)


if a_ents:
    with open(a_ents, 'r') as ents_file:
        ents = yaml.load(ents_file, Loader=yaml.CLoader)
        print(ents)
        for ent in ents['entities']:
            rels = ent.get("relations", [])
            base_ents = ['Person', 'Institution', 'Place', 'Work', 'Event']
            if isinstance(rels, str):
                if rels == 'all':
                    rels = base_ents + [x['name'].title() for x in ents['entities']]
            else:
                rels = base_ents + rels
            for r2 in rels:
                attributes = {"__module__":__name__}
                if r2 in base_ents:
                    rel_class_name = f"{r2.title()}{ent['name'].title()}"
                else:
                    rel_class_name = f"{ent['name'].title()}{r2.title()}"
                if rel_class_name not in globals().keys():
                    print(rel_class_name)
                    ent_class = type(rel_class_name, (AbstractRelation,), attributes)
                    globals()[rel_class_name] = ent_class


from django.conf.urls import url

from .tei_ac import TeiEntAc, TeiCompleterAc

app_name = 'apis_tei'

urlpatterns = [
    url(
        r'^autocomplete/(?P<entity>[a-zA-Z0-9-]+)/$',
        TeiEntAc.as_view(),
        name='generic_entities_autocomplete'
    ),
    url(
        r'^tei-completer/(?P<entity>[a-zA-Z0-9-]+)/$',
        TeiCompleterAc.as_view(),
        name='tei_completer_autocomplete'
    ),
]



from django.contrib.auth.decorators import login_required
from django.urls import reverse_lazy
from django.utils.decorators import method_decorator
from django.views.generic.detail import DetailView
from django.views.generic.edit import DeleteView

from browsing.browsing_utils import GenericListView, BaseCreateView, BaseUpdateView
from .filters import UriListFilter
from .forms import UriFilterFormHelper, UriForm
from .models import Uri
from .tables import UriTable


class UriListView(GenericListView):
    model = Uri
    filter_class = UriListFilter
    formhelper_class = UriFilterFormHelper
    table_class = UriTable
    init_columns = [
        'id',
        'uri',
        'entity',
    ]
    enable_merge = True


class UriDetailView(DetailView):
    model = Uri
    template_name = 'apis_metainfo/uri_detail.html'


class UriCreate(BaseCreateView):

    model = Uri
    form_class = UriForm

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(UriCreate, self).dispatch(*args, **kwargs)


class UriUpdate(BaseUpdateView):

    model = Uri
    form_class = UriForm

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(UriUpdate, self).dispatch(*args, **kwargs)


class UriDelete(DeleteView):
    model = Uri
    template_name = 'webpage/confirm_delete.html'
    success_url = reverse_lazy('apis_core:apis_metainfo:uri_browse')

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(UriDelete, self).dispatch(*args, **kwargs)



import re

from django.contrib.contenttypes.models import ContentType

from apis_core.apis_metainfo.models import Text
from apis_highlighter.models import Annotation


def highlight_text_new(*args, **kwargs):
    ann_proj_pk = kwargs.pop("set_ann_proj", False)
    types = kwargs.pop("types", False)
    users_show = kwargs.pop("users_show", False)
    inline_annotations = kwargs.pop("inline_annotations", True)
    t_start = 0
    t_end = 0
    obj = args[-1]
    if isinstance(obj, str):
        obj = Text.objects.get(pk=obj)
    lst_annot = []
    queries = dict()
    if users_show:
        queries["user_added_id__in"] = users_show
    if ann_proj_pk:
        queries["annotation_project__pk"] = ann_proj_pk
    queries["text"] = obj
    anns1 = Annotation.objects.filter(**queries).order_by("start")
    anns_fin = []
    for ann in anns1:
        for lb in re.finditer(r"[\r\n]", obj.text):
            if lb.start() < ann.start + (lb.end() - lb.start()):
                ann.start += lb.end() - lb.start()
                ann.end += lb.end() - lb.start()
        anns_fin.append(ann)
    for an in anns_fin:
        if types:
            test = False
            for m in an.entity_link.all():
                t = ContentType.objects.get_for_model(m)
                if str(t.pk) in types:
                    test = True
            if not test:
                continue
        c_start = re.findall(r"[\r\n]+", obj.text[: an.start])
        if len(c_start) > 0:
            an.start += len("".join(c_start))
        c_end = re.findall(r"[\r\n]+", obj.text[: an.end])
        if len(c_end) > 0:
            an.end += len("".join(c_end))
        if an.start >= t_start and an.start <= t_end:
            lst_annot[-1].append(an)
        else:
            lst_annot.append(
                [
                    an,
                ]
            )
        t_start = an.start
        t_end = an.end
    if len(lst_annot) == 0:
        html_return = obj.text
        html_return, nmbs = re.subn(r"\r\n", "<br/>", html_return)
        html_return, nmbs = re.subn(r"\r", "<br/>", html_return)
        html_return, nmbs = re.subn(r"\n", "<br/>", html_return)
        return html_return, None
    html_return = obj.text[: lst_annot[0][0].start]
    end = ""
    lst_end = None
    res_annotations = []
    for an in lst_annot:
        start = min([x.start for x in an])
        end = max([x.end for x in an])
        if len(an) > 1:
            start_span = """<mark class="highlight hl_text_complex" data-hl-type="complex" data-hl-start="{}" data-hl-end="{}" data-hl-text-id="{}">""".format(
                start, end, obj.pk
            )
            for an2 in an:
                res_annotations.append(an2.get_html_markup(include_object=True))
        else:
            start_span, res_ann = an[0].get_html_markup(include_object=True)
            res_annotations.append(res_ann)
        if lst_end:
            html_return += (
                obj.text[lst_end:start] + start_span + obj.text[start:end] + "</mark>"
            )
        else:
            html_return += start_span + obj.text[start:end] + "</mark>"
        lst_end = end
    html_return += obj.text[end:]
    if obj.text[0] == "\n":
        html_return = "-" + html_return[1:]
    if not inline_annotations:
        html_return = obj.text
    html_return, nmbs = re.subn(r"\r\n", "<br/>", html_return)
    html_return, nmbs = re.subn(r"\r", "<br/>", html_return)
    html_return, nmbs = re.subn(r"\n", "<br/>", html_return)
    return html_return, res_annotations


def highlight_text(*args, **kwargs):
    ann_proj_pk = kwargs.pop("set_ann_proj", False)
    types = kwargs.pop("types", False)
    users_show = kwargs.pop("users_show", False)
    t_start = 0
    t_end = 0
    obj = args[-1]
    if isinstance(obj, str):
        obj = Text.objects.get(pk=obj)
    if not types or not users_show:
        return obj.text
    lst_annot = []
    queries = dict()
    if users_show:
        queries["user_added_id__in"] = users_show
    if ann_proj_pk:
        queries["annotation_project__pk"] = ann_proj_pk
    queries["text"] = obj
    anns1 = Annotation.objects.filter(**queries).order_by("start")
    anns_fin = []
    for ann in anns1:
        for lb in re.finditer(r"[\r\n]", obj.text):
            if lb.start() < ann.start + (lb.end() - lb.start()):
                ann.start += lb.end() - lb.start()
                ann.end += lb.end() - lb.start()
        anns_fin.append(ann)
    for an in anns_fin:
        if types:
            test = False
            for m in an.entity_link.all():
                t = ContentType.objects.get_for_model(m)
                if str(t.pk) in types:
                    test = True
            if not test:
                continue

        if an.start >= t_start and an.start <= t_end:
            lst_annot[-1].append(an)
        else:
            lst_annot.append(
                [
                    an,
                ]
            )
        t_start = an.start
        t_end = an.end
    if len(lst_annot) == 0:
        html_return = obj.text
        html_return, nmbs = re.subn(r"\r\n", "<br/>", html_return)
        html_return, nmbs = re.subn(r"\r", "<br/>", html_return)
        html_return, nmbs = re.subn(r"\n", "<br/>", html_return)
        return html_return
    html_return = obj.text[: lst_annot[0][0].start]
    end = ""
    lst_end = None
    for an in lst_annot:
        start = min([x.start for x in an])
        end = max([x.end for x in an])
        if len(an) > 1:
            start_span = """<mark class="highlight hl_text_complex" data-hl-type="complex" data-hl-start="{}" data-hl-end="{}" data-hl-text-id="{}">""".format(
                start, end, obj.pk
            )
        else:
            start_span = an[0].get_html_markup()
        if lst_end:
            html_return += (
                obj.text[lst_end:start] + start_span + obj.text[start:end] + "</mark>"
            )
        else:
            html_return += start_span + obj.text[start:end] + "</mark>"
        lst_end = end
    html_return += obj.text[end:]
    if obj.text[0] == "\n":
        html_return = "-" + html_return[1:]

    html_return, nmbs = re.subn(r"\r\n", "<br/>", html_return)
    html_return, nmbs = re.subn(r"\r", "<br/>", html_return)
    html_return, nmbs = re.subn(r"\n", "<br/>", html_return)
    return html_return


def highlight_textTEI(*args, **kwargs):
    user_pk = kwargs.pop("user", False)
    ann_proj_pk = kwargs.pop("ann_proj", False)
    obj = args[-1]
    t_start = 0
    t_end = 0
    if isinstance(obj, str):
        obj = Text.objects.get(pk=obj)
    lst_annot = []
    for an in Annotation.objects.filter(text=obj).order_by("start"):
        if an.start >= t_start and an.start <= t_end:
            lst_annot[-1].append(an)
        else:
            lst_annot.append(
                [
                    an,
                ]
            )
        t_start = an.start
        t_end = an.end
    # print(lst_annot)
    if len(lst_annot) == 0:
        return obj.text
    html_return = obj.text[: lst_annot[0][0].start]
    end = ""
    lst_end = None
    for an in lst_annot:
        start = min([x.start for x in an])
        end = max([x.end for x in an])
        try:
            lst_classes = " ".join(
                [str(x.relation_type.pk) for x in an[0].entity_link.all()]
            )
        except:
            try:
                lst_classes = " ".join(
                    [str(x.kind.pk) for x in an[0].entity_link.all()]
                )
            except:
                lst_classes = ""
        if len(an) > 1:
            start_span = '<name type="complex" hl-start="{}" hl-end="{}" hl-text-id="{}">'.format(
                start, end, obj.pk
            )
        else:
            try:
                entity_type = type(an[0].entity_link.all()[0]).__name__
                entity_pk = an[0].entity_link.all()[0].pk
                ent_lst_pk = []
                for x in dir(an[0].entity_link.all()[0]):
                    c = re.match("related_\w+_id", x)
                    if c:
                        ent_lst_pk.append(
                            str(getattr(an[0].entity_link.all()[0], c.group(0)))
                        )
                if len(ent_lst_pk) == 0:
                    ent_lst_pk.append(str(an[0].entity_link.all()[0].pk))
            except:
                entity_type = ""
                entity_pk = ""
                ent_lst_pk = []
            try:
                entity_uri = (
                    an[0].entity_link.all()[0].uri_set.values_list("uri", flat=True)[0]
                )
            except:
                entity_uri = "internal db id: {}".format(an[0].entity_link.all()[0].pk)
            start_span = '<name hl-type="simple" hl-start="{}" hl-end="{}" hl-text-id="{}" hl-ann-id="{}" type="{}" entity-pk="{}" related-entity-pk="{}">'.format(
                start,
                end,
                obj.pk,
                an[0].pk,
                entity_type,
                entity_pk,
                ",".join(ent_lst_pk),
            )
            #'<span class="highlight hl_text_{}" data-hl-type="simple" data-hl-start="{}" data-hl-end="{}" data-hl-text-id="{}" data-hl-ann-id="{}" data-entity-class="{}" data-entity-pk="{}" data-related-entity-pk="{}">'.format(start, end, obj.pk, an[0].pk, entity_type, entity_pk, ','.join(ent_lst_pk))
        if lst_end:
            if len(an) > 1:
                html_return += (
                    obj.text[lst_end:start]
                    + start_span
                    + obj.text[start:end]
                    + "</name>"
                )
            else:
                html_return += (
                    obj.text[lst_end:start]
                    + start_span
                    + obj.text[start:end]
                    + "<index><term>"
                    + entity_uri
                    + "</term></index></name>"
                )
        else:
            html_return += start_span + obj.text[start:end] + "</name>"
        lst_end = end
    html_return += obj.text[end:]
    return html_return
